package cn.net.connor.process;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

@SuppressWarnings("serial")
public class KLocalizer extends JFrame{

	private JTextField tf_key;
	private JTextField tf_en;
	private JTextField tf_cn;
	private JButton b_add;
	private JLabel l_info;
	private JButton b_close;
	
	public static void main(String[] args) {
		new KLocalizer().initUI();
	}

	public void initUI() {
		this.setTitle("国际化");
		this.setMinimumSize(new Dimension(400,270));
		this.setPreferredSize(new Dimension(400,270));
		this.setLayout(new BorderLayout());
		this.add(BorderLayout.CENTER,initCenterPanel());
		this.add(BorderLayout.SOUTH,initBtnPanel());
		addListeners();
		pack();
		this.setLocationRelativeTo(null);
		this.setAlwaysOnTop(true);
		setVisible(true);
	}

	private void addListeners() {
		b_add.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					addLocaleInfo();
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			}
		});
		this.b_close.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
	}

	protected void addLocaleInfo() throws Exception{
		this.l_info.setText("");
		String key = this.tf_key.getText();
		String en = this.tf_en.getText();
		String cn = this.tf_cn.getText();
		Package p = this.getClass().getPackage();
		String path = "src/"+p.getName().replace('.', '/');
		System.out.println("PATH: "+path);
		int index = path.lastIndexOf('/');
		if(index<=0) {
			this.l_info.setText("类路径不正确");
			return;
		}
		String title = path.substring(index+1);
		String enFile = path+"/"+title+"_locale.properties";
		String cnFile = path+"/"+title+"_locale_zh_CN.properties";
		System.out.println("TITLE: "+title);
		if(title.length()>0) {
			String code = title.substring(0,1).toUpperCase()+title.substring(1)+"Locale.getString(\""+key+"\")";
			System.out.println("PASTE: "+code);
			StringSelection selection = new StringSelection(code);
			Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
			clipboard.setContents(selection, selection);
		}
		File file = new File(enFile);
		if(!file.exists()) {
			file.createNewFile();
		}
		file = new File(cnFile);
		if(!file.exists()) {
			file.createNewFile();
		}
		//System.out.println("英文路径："+enFile);
		//System.out.println("中文路径："+cnFile);
		Properties enProps = new Properties();
		enProps.load(new FileReader(enFile));
		Properties cnProps = new Properties();
		cnProps.load(new FileReader(cnFile));
		if(enProps.containsKey(key)||cnProps.containsKey(key)) {
			this.l_info.setText("关键字“"+key+"”已存在");
			System.out.println(title.substring(0,1).toUpperCase()+title.substring(1)+"Locale.getString(\""+key+"\")");
			return;
		}
		if(isEmpty(key)||isEmpty(en)||isEmpty(cn)) {
			this.l_info.setText("请填写完整信息");
			return;
		}
		System.out.println("英文："+key+"="+en);
		System.out.println("中文："+key+"="+cn);
		enProps.setProperty(key, en);
		cnProps.setProperty(key, cn);
		FileOutputStream enos;
		enProps.store(enos = new FileOutputStream(enFile), "");
		FileOutputStream cnos;
		cnProps.store(cnos = new FileOutputStream(cnFile), "");
		enos.flush();
		enos.close();
		cnos.flush();
		cnos.close();
		this.l_info.setText("写入成功");
		tf_key.setText("");
		tf_en.setText("");
		tf_cn.setText("");
	}

	private JPanel initBtnPanel() {
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER,15,5));
		panel.add(this.b_add = new JButton(" 添加 "));
		panel.add(this.b_close = new JButton(" 关闭 "));
		return panel;
	}

	private JPanel initCenterPanel() {
		JPanel panel = new JPanel(new GridBagLayout());
		panel.setBorder(BorderFactory.createEmptyBorder(0,30,0,25));
		GridBagConstraints s = new GridBagConstraints(0, 0, 1, 1, 0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(4, 0, 4, 5), 0, 0);
		newRow(panel, new JLabel("关键字: "), s, 1);
		addCol(panel, this.tf_key = new JTextField(), s, 1, 1);
		newRow(panel, new JLabel("英文: "), s, 1);
		addCol(panel, this.tf_en = new JTextField(), s, 1, 1);
		newRow(panel, new JLabel("中文: "), s, 1);
		addCol(panel, this.tf_cn = new JTextField(), s, 1, 1);
		this.l_info = new JLabel();
		this.l_info.setForeground(Color.RED);
		this.l_info.setPreferredSize(new Dimension(100,50));
		newRow(panel, l_info, s, 2);
		return panel;
	}
	
	public static boolean isEmpty(String str) {
		return str==null||"".equals(str.trim());
	}

	@Override
	protected void processWindowEvent(WindowEvent e) {
		if(e.getID()==WindowEvent.WINDOW_CLOSING) {
			dispose();
		}
		super.processWindowEvent(e);
	}
	
	public static void newRow(JPanel panel, Component comp, GridBagConstraints s, int width) {
		s.weightx = 0;
		s.gridwidth = width;
		s.gridx = 0;
		s.gridy++;
		panel.add(comp, s);
	}

	public static void addCol(JPanel panel, Component comp, GridBagConstraints s, int width, double weightx) {
		s.gridx++;
		s.gridwidth = width;
		s.weightx = weightx;
		panel.add(comp, s);
	}
}
