package cn.net.connor.process.gcjlk;

import java.io.File;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.filechooser.FileSystemView;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.AIFComponentContext;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.aifrcp.AIFUtility;
import com.teamcenter.rac.kernel.TCComponentBOMLine;
import com.teamcenter.rac.kernel.TCComponentBOPWindow;
import com.teamcenter.rac.kernel.TCComponentBOPWindowType;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCException;
import com.teamcenter.rac.util.MessageBox;
import com.teamcenter.rac.util.Registry;

import cn.net.connor.process.ProcessUtil;
import k.KDialogController;
import k.KOperation2;
import k.util.KSqlUtil;
import k.util.KUtil;
import k.word.KWord;
import k.word.KWordUtil;

public class GCJLKDialogController extends KDialogController {

	private TCComponentItemRevision targetRev;
	private TCComponentItemRevision procRev;
	protected String targetItemID = "";
	protected String targetRevID = "";
	private GCJLKDialog dialog;
	private String dbUrl;
	private String dbUser;
	private String dbPwd;
	private String sql;
	private KWord kWord;
	private static final Registry REG = Registry.getRegistry(GCJLKDialogController.class);
	private static final String PREF_QUERY_GCJLK = "CONNOR_AVIC_GCJLK_Query";
	private static final String PREF_EXPORT_GCJLK = "CONNOR_AVIC_GCJLK_Export";

	public GCJLKDialogController(AbstractAIFApplication app) {
		super(app);
	}

	@Override
	public boolean init() throws Exception {
		this.dialog = (GCJLKDialog) aifDialog;
		InterfaceAIFComponent target = app.getTargetComponent();
		System.out.println("Select: " + target);
		if (target != null && (target instanceof TCComponentItemRevision)) {
			this.targetRev = (TCComponentItemRevision) target;
		}
		if (this.targetRev == null) {
			throw new TCException(REG.getString("selectrev.WARN"));
		}
		AIFComponentContext[] proc = targetRev.whereReferencedByTypeRelation(null, new String[] { "IMAN_METarget" });
		System.out.println("proc: " + targetRev + " -> " + Arrays.toString(proc));
		int procCnt = KUtil.getLen(proc);
		for (int i = 0; i < procCnt; i++) {
			InterfaceAIFComponent comp = proc[i].getComponent();
			if (comp instanceof TCComponentItemRevision) {
				if (ProcessUtil.isMEProcess(((TCComponentItemRevision) comp).getItem().getType())) {
					this.procRev = (TCComponentItemRevision) comp;
				}
			}
		}
		System.out.println("find process: " + this.procRev);
		if (this.procRev == null) {
			throw new TCException(REG.getString("noprocess.ERROR"));
		}
		this.targetItemID = this.targetRev.getStringProperty("item_id");
		this.targetRevID = this.targetRev.getStringProperty("item_revision_id");
		String[] prefValues = KUtil.getPrefVals(session, PREF_QUERY_GCJLK);
		int len = KUtil.getLen(prefValues);
		if (len < 4) {
			throw new TCException(String.format(REG.getString("pref.ERROR1"), PREF_QUERY_GCJLK));
		}
		this.dbUrl = prefValues[0];
		this.dbUser = prefValues[1];
		this.dbPwd = prefValues[2];
		this.sql = "";
		for (int i = 3; i < len; i++) {
			sql += prefValues[i];
			if (i != len - 1) {
				sql += "\n";
			}
		}
		System.out.println("SQL: " + sql);
		this.kWord = new KWord(session, PREF_EXPORT_GCJLK);
		return true;
	}

	public void doQuery() throws Exception {
		String itemID = dialog.tf_ItemID.getText();
		// String revID = dialog.tf_RevID.getText();
		System.out.println(String.format("Query - [%s]", itemID));
		if (KUtil.isEmpty(itemID)) {
			throw new TCException(REG.getString("queryinput.ERROR"));
		}
		Connection conn = KSqlUtil.getSqlServerConnection(dbUrl, dbUser, dbPwd);
		try {
			dialog.tableModel.getDataVector().clear();
			List<String[]> data = KSqlUtil.queryStringArray(conn, sql, new String[] { itemID }, 3);
			int rowCnt = data == null ? 0 : data.size();
			System.out.println("query result count: " + rowCnt);
			for (int i = 0; i < rowCnt; i++) {
				String[] rowData = data.get(i);
				dialog.tableModel.addRow(new Object[] { i + 1, rowData[0], rowData[1], rowData[2] });
			}
		} finally {
			KSqlUtil.free(conn);
		}
	}

	public void doExport() throws Exception {
		int rowNum = dialog.table.getSelectedRow();
		System.out.println("select row: " + rowNum);
		if (rowNum < 0) {
			throw new TCException(REG.getString("selectrow.ERROR"));
		}
		String fileName = procRev.getStringProperty("item_id")+REG.getString("filename")+KUtil.getDateStr(new Date(), "_yyyyMMdd")+".docx";
		FileSystemView fsv = FileSystemView.getFileSystemView();
		String deskPath = fsv.getHomeDirectory().getPath();
		String defaultFile = deskPath+File.separator+fileName;
		File exportFile = KUtil.chooseSaveFile(dialog, defaultFile, "docx");
		if(exportFile==null) {
			System.out.println("cancel export");
			return;
		}
		String orderNumber = (String) dialog.table.getValueAt(rowNum, 1);
		String quantity = (String) dialog.table.getValueAt(rowNum, 2);
		String isFirst = (String) dialog.table.getValueAt(rowNum, 3);
		Map<String, String> overwriteTexts = new HashMap<String, String>();
		overwriteTexts.put(dialog.header[1], orderNumber);
		overwriteTexts.put(dialog.header[2], quantity);
		overwriteTexts.put(dialog.header[3], isFirst);
		System.out.println("Overwrite texts: " + overwriteTexts);
		dialog.disposeDialog();
		new KOperation2(AIFUtility.getCurrentApplication(), REG.getString("export.STATUS")) {

			private TCComponentBOMLine procLine;
			private TCComponentBOPWindow bopWindow;

			@Override
			public boolean init() throws Exception {
				TCComponentBOPWindowType bopWindowType = (TCComponentBOPWindowType) session.getTypeComponent("BOPWindow");
				bopWindow = bopWindowType.createBOPWindow(null);
				procLine = bopWindow.setWindowTopLine(procRev.getItem(), procRev, null, null);
				return true;
			}

			@Override
			public void execute() throws Exception {
				try {
					List<KWord> kWords = new ArrayList<KWord>();
					kWords.add(kWord);
					List<TCComponentBOMLine> targetLines = new ArrayList<TCComponentBOMLine>();
					List<TCComponentItemRevision> meTargetRevs = new ArrayList<TCComponentItemRevision>();
					targetLines.add(procLine);
					meTargetRevs.add(targetRev);
					KWordUtil.startWord();
					try {
						KWord.export(session, kWords, REG.getString("filename"), null, targetLines, meTargetRevs, true, overwriteTexts, exportFile);
						KUtil.info(dialog, REG.getString("export.SUCCESS"));
					} finally {
						KWordUtil.closeWord();
					}
				} catch (Exception e) {
					e.printStackTrace();
					MessageBox.post(dialog, e);
				}
			}

			@Override
			public void clearCache() throws Exception {
				if (bopWindow != null) {
					bopWindow.refresh();
					bopWindow.setClearCacheOnClose(true);
					bopWindow.closeBOPWindow();
				}
			}
		}.executeModeless();

	}

}
