package cn.net.connor.process.gzbomexport;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.JOptionPane;

import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.AIFComponentContext;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentBOMLine;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.pse.services.PSEApplicationService;

import cn.net.connor.process.ProcessLocale;
import k.KOperation;
import k.util.KUtil;
import k.util.POIUtil;

public class GZBomExportOperation extends KOperation{
	
	private static final String FILENAME = ProcessLocale.getString("gzbomfilename");
	private static final String PREF_GZBOMEXPORT = "Connor_GZBomExport";
	private TCComponentBOMLine topLine;
	private List<Config> configs = new ArrayList<GZBomExportOperation.Config>();
	private int rowNum = 2;
	private XSSFSheet sheet;
	private XSSFCellStyle cellStyle;
	private static final int STARTCOL = 0;
	private String dateformat = "_yyyy_MM_dd";

	public GZBomExportOperation(AbstractAIFApplication app, String opName) {
		super(app, opName);
	}
	
	public class Config{
		String propName;
		boolean isBomProp;
		public Config(String propName, boolean isBomProp) {
			this.propName = propName;
			this.isBomProp = isBomProp;
		}
	}

	@Override
	public boolean init() throws Exception {
		if(!(app instanceof PSEApplicationService)) {
			KUtil.error(null, ProcessLocale.getString("notinpse.ERROR"));
			return false;
		}
		String[] prefs = KUtil.getPrefVals(session, PREF_GZBOMEXPORT);
		int len = KUtil.getLen(prefs);
		for(int i=0;i<len;i++) {
			String pref = prefs[i].trim();
			if(pref.length()==1&&Character.isDigit(pref.charAt(0))) {
				int num = pref.charAt(0)-'0';
				if(num>=0) {
					System.out.println("RESET Start Row: "+num);
					this.rowNum = num;
				}
				continue;
			}
			if(pref.startsWith("##")) {
				this.dateformat = pref.substring(2);
				System.out.println("RESET DATE FORMAT: "+this.dateformat);
				continue;
			}
			String propName;
			boolean isBomProp = pref.startsWith("#");
			if(isBomProp) {
				propName = pref.substring(1);
			}else {
				propName = pref;
			}
			this.configs.add(new Config(propName, isBomProp));
		}
		if(this.configs.size()==0) {
			KUtil.info(null, String.format(ProcessLocale.getString("prefnotconfiged.ERROR1"), PREF_GZBOMEXPORT));
			return false;
		}
		PSEApplicationService pse = (PSEApplicationService) app;
		this.topLine = pse.getTopBOMLine();
		if(topLine==null) {
			KUtil.info(null, ProcessLocale.getString("notopline.ERROR"));
			return false;
		}
		System.out.println("Export: "+topLine);
		return true;
	}
	
	private void exportLine(TCComponentBOMLine parentLine) throws Exception{
		XSSFRow row = POIUtil.getRow(sheet, rowNum++);
		TCComponentItemRevision rev = parentLine.getItemRevision();
		TCComponent form = rev.getRelatedComponent("IMAN_master_form_rev");
		int colCnt = this.configs.size();
		for(int i=0;i<colCnt;i++) {
			int col = STARTCOL+i;
			Config config = this.configs.get(i);
			String val;
			if(KUtil.isEmpty(config.propName)) {
				val = "";
			}else {
				val = config.isBomProp? parentLine.getProperty(config.propName):(form==null?"":form.getProperty(config.propName));
			}
			POIUtil.setValue(row, col, val, cellStyle);
		}
		AIFComponentContext[] cLines = parentLine.getChildren();
		int len = KUtil.getLen(cLines);
		for(int i=0;i<len;i++) {
			TCComponentBOMLine cLine = (TCComponentBOMLine) cLines[i].getComponent();
			exportLine(cLine);
		}
	}

	@Override
	public void execute() throws Exception {
		String time = KUtil.isEmpty(dateformat)?"":KUtil.getDateStr(new Date(), dateformat);
		File file = KUtil.chooseSaveFile(((PSEApplicationService) app).getApplicationPanel(), FILENAME+time+".xlsx");
		if(file==null) {
			System.out.println("Cancel Export");
			return;
		}
		System.out.println("Save File: "+file.getAbsolutePath());
		InputStream input = this.getClass().getResourceAsStream("template.xlsx");
		XSSFWorkbook wb = new XSSFWorkbook(input);
		this.cellStyle = POIUtil.getCellStyle(wb, null, false);
		input.close();
		this.sheet = wb.getSheetAt(0);
		exportLine(topLine);
		FileOutputStream os = new FileOutputStream(file);
		wb.write(os);
		os.flush();
		os.close();
		int op = JOptionPane.showOptionDialog(((PSEApplicationService) app).getApplicationPanel(), ProcessLocale.getString("exportsuccess.INFO"), "", JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE, null, null, null);
		if(op==JOptionPane.YES_OPTION) {
			KUtil.openFile(file);
		}
	}

}
