package cn.net.connor.process.meopmodel;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentBOMLine;
import com.teamcenter.rac.kernel.TCComponentBOPLine;
import com.teamcenter.rac.kernel.TCComponentDataset;
import com.teamcenter.rac.kernel.TCComponentItem;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCComponentItemType;
import com.teamcenter.rac.kernel.TCException;
import com.teamcenter.rac.util.Registry;

import cn.net.connor.process.ProcessUtil;
import k.KOperation2;
import k.util.KUtil;

public class DesignModelOperation extends KOperation2 {

	private TCComponentBOPLine meopLine;
	private TCComponentItemType itemType;
	private TCComponentBOMLine processLine;
	private TCComponentItemRevision meTargetRev;
	private List<TCComponent> datasets;
	public static final String TYPE_ITEM = "Ac5_GXMX";
	public static final String REL_REV_DATASET = "IMAN_specification";
	public static final String TYPE_DATASET = "CATPart";
	private static final Registry REG = Registry.getRegistry(DesignModelOperation.class);

	public DesignModelOperation(AbstractAIFApplication app, String opName, TCComponentBOPLine meopLine) {
		super(app, opName);
		this.meopLine = meopLine;
	}

	@Override
	public boolean init() throws Exception {
		if (this.meopLine == null) {
			throw new TCException("meopLine can not be null");
		}
		try {
			this.itemType = (TCComponentItemType) session.getTypeComponent(TYPE_ITEM);
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (this.itemType == null) {
			throw new TCException("Type is not defined: " + TYPE_ITEM);
		}
		TCComponentBOMLine pLine = this.meopLine.parent();
		while (pLine != null) {
			if (ProcessUtil.isMEProcess(pLine.getItem().getType())) {
				this.processLine = pLine;
				break;
			}
			pLine = pLine.parent();
		}
		System.out.println("Process line: " + this.processLine);
		if (this.processLine == null) {
			throw new TCException(REG.getString("meprocessnotfound.ERROR"));
		}
		TCComponent[] meTargetRevs = this.processLine.getItemRevision().getRelatedComponents("IMAN_METarget");
		System.out.println("METargets: " + Arrays.toString(meTargetRevs));
		this.meTargetRev = getLatestReleasedRev(meTargetRevs);
		System.out.println("Latest Released Rev: " + meTargetRev);
		if (this.meTargetRev == null) {
			throw new TCException(REG.getString("releasedmetargetnotfound.ERROR"));
		}
		this.datasets = KUtil.getChildComponents(meTargetRev, TYPE_DATASET, REL_REV_DATASET);
		System.out.println("Datasets: " + datasets);
		return true;
	}

	public static TCComponentItemRevision getLatestReleasedRev(TCComponent[] comps) {
		int size = comps == null ? 0 : comps.length;
		if (size == 0) {
			return null;
		}
		TCComponentItemRevision res = null;
		Date maxDate = null;
		try {
			for (int i = 0; i < size; i++) {
				TCComponent temp = comps[i];
				if (!(temp instanceof TCComponentItemRevision)) {
					continue;
				}
				Date tempDate = temp.getDateProperty("date_released");
				if (tempDate == null) {
					continue;
				}
				if (maxDate == null || tempDate.after(maxDate)) {
					maxDate = tempDate;
					res = (TCComponentItemRevision) temp;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return res;
	}

	@Override
	public void execute() throws Exception {
		String objectName = this.meTargetRev.getStringProperty("object_name") + REG.getString("name.SUFFIX");
		String itemID = itemType.getNewID();
		String revID = itemType.getNewRev(null);
		TCComponentItem newItem = itemType.create(itemID, revID, TYPE_ITEM, objectName, "", null);
		System.out.println("New item: " + newItem);
		newItem.refresh();
		TCComponentItemRevision newRev = newItem.getLatestItemRevision();
		System.out.println("New rev: " + newRev);
		newRev.refresh();
		TCComponentBOMLine newLine = this.meopLine.add(newItem, newRev, null, false);
		System.out.println("New line: " + newLine);
		int datasetCnt = this.datasets == null ? 0 : this.datasets.size();
		for (int i = 0; i < datasetCnt; i++) {
			TCComponentDataset dataset = (TCComponentDataset) datasets.get(i);
			TCComponentDataset newDataset = dataset.saveAs(dataset.getStringProperty("object_name"));
			System.out.println("New dataset: " + newDataset);
			newRev.add(REL_REV_DATASET, newDataset);
		}
		newRev.refresh();
		this.meopLine.refresh();
	}

	@Override
	public void clearCache() throws Exception {

	}

}
