package cn.net.connor.process.meopviewfx.ui;

import javax.swing.JTextArea;

import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import com.teamcenter.rac.stylesheet.gdt.GdtDialog;
import com.teamcenter.rac.stylesheet.gdt.GdtEditor;
import com.teamcenter.rac.stylesheet.gdt.GenerateGDTHTML;
import com.teamcenter.rac.util.UIUtilities;

import cn.net.connor.process.meopviewfx.bean.KTableRow;
import cn.net.connor.process.meopviewfx.bean.PropConfig;
import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Worker.State;
import javafx.event.Event;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableColumn.CellEditEvent;
import javafx.scene.control.TablePosition;
import javafx.scene.control.TableRow;
import javafx.scene.control.TableView;
import javafx.scene.input.MouseButton;
import javafx.scene.layout.Region;
import javafx.scene.web.WebView;

/**
 * select color:
 * 
 * .table-row-cell .ktext { -fx-fill: black; }
 * 
 * .table-row-cell:selected .ktext { -fx-fill: white; }
 * 
 * .table-cell .ktext { -fx-fill: black; }
 * 
 * .table-cell:selected .ktext { -fx-fill: white; }
 * 
 * @author Administrator
 *
 * @param <T>
 * @param <E>
 */

public class KFXEditableStringAreaHtmlTableCell<T, E> extends TableCell<T, String> {
	private WebView webView;
	private PropConfig config;

	public KFXEditableStringAreaHtmlTableCell(PropConfig config) {
		super();
		this.config = config;
	}

	@Override
	public void startEdit() {
		super.startEdit();
	}

	@Override
	public void cancelEdit() {
		super.cancelEdit();
		// setText((String) getItem());
		// setGraphic(null);
		setGraphic(webView);
	}

	@Override
	public void updateItem(String item, boolean empty) {
		super.updateItem(item, empty);
		if (webView == null) {
			createWebView();
		}
		if (empty) {
			setText(null);
			setGraphic(null);
		} else {
			webView.setPrefHeight(Region.USE_COMPUTED_SIZE);
			String newValue = item;
			Platform.runLater(() -> {
				try {
					// System.out.println("Load Html: " + newValue);
					String html = newValue == null || newValue.startsWith("<html>") ? newValue : GenerateGDTHTML.generatePage(newValue);
					// System.out.println(">> " + html);
					webView.getEngine().loadContent(html);
				} catch (Exception e) {
					e.printStackTrace();
				}
			});
			setGraphic(webView);
		}
	}

	private void createWebView() {
		webView = new WebView();
		webView.setPrefHeight(Region.USE_COMPUTED_SIZE);
		webView.getEngine().getLoadWorker().stateProperty().addListener(new ChangeListener<State>() {
			@Override
			public void changed(ObservableValue<? extends State> observable, State oldValue, State newValue) {
				if (newValue != State.SUCCEEDED) {
					return;
				}
				try {
					Object res = webView.getEngine().executeScript(
							"var body = document.body, html = document.documentElement; Math.max( body.scrollHeight, body.offsetHeight, html.clientHeight, html.scrollHeight, html.offsetHeight );");
					// System.out.println("Height: " + res);
					int height = Integer.parseInt(res.toString());
					// int width = Integer.parseInt(webView.getEngine().executeScript("var body =
					// document.body, html = document.documentElement; Math.max( body.scrollWidth,
					// body.offsetWidth, html.clientWidth, html.scrollWidth, html.offsetWidth
					// );").toString());
					webView.setPrefHeight(height+5);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
		if (isEditable()) {
			webView.setOnMouseClicked(e -> {
				if (e.getButton() != MouseButton.PRIMARY || e.getClickCount() != 2) {
					return;
				}
				TableRow<?> row = getTableRow();
				KTableRow bean = (KTableRow) getTableView().getItems().get(row.getIndex());
				String oldValue = bean.stringProps.get(config).getValueSafe();
				System.out.println("Modify [" + row + "]: " + oldValue);
				Display localDisplay = Display.getCurrent();
				if (localDisplay == null) {
					localDisplay = Display.getDefault();
				}
				GdtEditor editor = new GdtEditor(new JTextArea(oldValue));
				editor.setValue(oldValue);
				localDisplay.asyncExec(new Runnable() {
					public void run() {
						Shell localShell = UIUtilities.getCurrentModalShell();
						GdtDialog gdtDialog = new GdtDialog(localShell, null, editor, true) {
							public boolean close() {
								try {
									String newValue = editor.getValue();
									System.out.println("New Value: " + newValue);
									bean.stringProps.get(config).setValue(newValue);

								} catch (Exception e) {
									e.printStackTrace();
								}
								return super.close();
							}
						};
						editor.setGdtDialog(gdtDialog);
						gdtDialog.open();
					}
				});
			});
		}
		// webView.prefWidthProperty().bind(getTableColumn().widthProperty().subtract(20));
	}

	protected String getString() {
		return getItem() == null ? "" : getItem().toString();
	}

	@Override
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public void commitEdit(String item) {
		if (isEditing()) {
			super.commitEdit(item);
		} else {
			final TableView table = getTableView();
			if (table != null) {
				TablePosition position = new TablePosition(getTableView(), getTableRow().getIndex(), getTableColumn());
				CellEditEvent editEvent = new CellEditEvent(table, position, TableColumn.editCommitEvent(), item);
				Event.fireEvent(getTableColumn(), editEvent);
			}
			updateItem(item, false);
			if (table != null) {
				table.edit(-1, null);
			}
			// super.commitEdit(item); // 20201217 bug fix
		}
	}

}
