package cn.net.connor.process.meopviewfx.ui;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.teamcenter.rac.aif.kernel.AIFComponentContext;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentDataset;
import com.teamcenter.rac.kernel.TCComponentFolder;

import cn.net.connor.process.ProcessLocale;
import cn.net.connor.process.meopviewfx.bean.TextBean;
import k.util.KSql;
import k.util.KSqlUtil;
import k.util.KUtil;

public class TextTreeLoader {

	private static final String PREF_COMMON_FOLDER = "Connor_AVIC_GXBZ_Common_Folder";
	private static final String PREF_PERSONAL_FOLDER = "Connor_AVIC_GXBZ_Personal_Folder";
	private static final String FLAG_REALVALUE = "#RV#";
	private static final String PREFFLAG_REALVALUE = "#RealValue#";

	public static class FileBean {
		File file;
		int realValueCol;

		public FileBean(File file, int realValueCol) {
			this.file = file;
			this.realValueCol = realValueCol;
		}

	}
	
	public Map<String, TextBean> loadText() throws Exception {
		Map<String, TextBean> textMap = new LinkedHashMap<String, TextBean>();
		String key = ProcessLocale.getString("mkhgy");
		//String sql = "select pac5_JG1, pac5_JG2, pac5_JG3, pac5_JG4, pac5_JG5, pac5_JG6 from pAc5_MKHGYRevMasters";
		String sql = "select pac5_JG1, pac5_JG2, pac5_JG3, pac5_JG4, pac5_JG5, pac5_JG6, t4.pitem_id, t3.pitem_revision_id from pAc5_MKHGYRevMasters t0\r\n" + 
				"left join pform t1 on t0.puid = t1.rdata_fileu\r\n" + 
				"left join (select * from pimanrelation where rrelation_typeu = (select puid from pimantype where ptype_name = 'IMAN_master_form')) t2 on t2.rsecondary_objectu = t1.puid\r\n" + 
				"left join pitemrevision t3 on t3.puid = t2.rprimary_objectu\r\n" + 
				"left join pitem t4 on t4.puid = t3.ritems_tagu";
		sql+="\r\nleft join (\r\n" + 
				"  select r1.puid from pAc5_MKHGYRevision r1\r\n" + 
				"  left join pitemrevision r2 on r2.puid = r1.puid\r\n" + 
				"  where \r\n" + 
				"  (select pdate_released from pworkspaceobject where puid = r2.puid) is not null\r\n" + 
				"  and (select pdate_released from pworkspaceobject where puid = r2.puid) = "
				+ "( select max(dr) from (select (select pdate_released from pworkspaceobject where puid = pitemrevision.puid) dr from pitemrevision where ritems_tagu = r2.ritems_tagu) )\r\n" + 
				"  and r2.psequence_id = (select max(psequence_id) from pitemrevision where ritems_tagu = r2.ritems_tagu and pitem_revision_id = r2.pitem_revision_id)\r\n" + 
				") t5 on t5.puid = t3.puid\r\n" + 
				"where (t0.pac5_Effictiveness is null or t0.pac5_Effictiveness!='" + ProcessLocale.getString("shixiao") + "') and t5.puid is not null";//失效
		System.out.println("sql:" + sql);
		Connection conn = KSqlUtil.getOracleConnection(KSql.PREF_DB_CONN);
		List<String[]> data = null;
		int colCnt = 8;
		TextBean root = new TextBean(key);
		textMap.put(key, root);
		try {
			data = KSqlUtil.queryStringArray(conn, sql, null, colCnt);
		} finally {
			KSqlUtil.free(conn);
		}
		int cnt = data == null?0:data.size();
		for(int i=0; i<cnt ;i++) {
			String[] rowData = data.get(i);
			TextBean parent = root;
			String itemId = rowData[colCnt-2];
			String revId = rowData[colCnt-1];
			for(int c=0; c<colCnt-2; c++) {
				String text = rowData[c];
				if(KUtil.isEmpty(text)) {
					break;
				}
				TextBean child = new TextBean(text, itemId, revId);
				int ind = parent.children.indexOf(child);
				if(ind>=0) {
					parent = parent.children.get(ind);
				}else {
					parent.children.add(child);
					parent = child;
				}
			}
		}
		return textMap;
	}
	
	

	public Map<String, TextBean> loadTextFromExcels() throws Exception {
		Map<String, TextBean> textMap = new LinkedHashMap<String, TextBean>();
		List<FileBean> docFiles = getConfigFile();
		int size = docFiles.size();
		if (size == 0) {
			System.out.println("Special Text File not found");
			return textMap;
		}
		for (int i = 0; i < size; i++) {
			FileBean docFile = docFiles.get(i);
			System.out.println("Load File: " + docFile.file.getAbsolutePath());
			InputStream in = new FileInputStream(docFile.file);
			XSSFWorkbook wb = new XSSFWorkbook(in);
			in.close();
			int sheetCnt = wb.getNumberOfSheets();
			for (int j = 0; j < sheetCnt; j++) {
				XSSFSheet sheet = wb.getSheetAt(0);
				String sheetName = sheet.getSheetName();
				System.out.println(">> Load Sheet: " + sheetName);
				if (textMap.containsKey(sheetName)) {
					System.out.println("SKIP");
					continue;
				}
				TextBean bean = readSheet(sheet, docFile.realValueCol);
				if (bean != null && bean.children.size() > 0) {
					textMap.put(sheetName, bean);
				} else {
					System.out.println("No Data");
				}
			}
		}
		return textMap;
	}

	private TextBean readSheet(XSSFSheet sheet, int realValueCol) throws Exception {
		TextBean root = new TextBean(sheet.getSheetName());
		int firstRow = sheet.getFirstRowNum();
		int lastRow = sheet.getLastRowNum();
		Map<Integer, TextBean> parents = new HashMap<Integer, TextBean>();
		parents.put(0, root);
		for (int i = firstRow; i <= lastRow; i++) {
			XSSFRow row = sheet.getRow(i);
			if (row == null) {
				continue;
			}
			int firstCol = row.getFirstCellNum();
			if (firstCol < 0) {
				firstCol = 0;
			}
			int lastCol = row.getLastCellNum();
			for (int j = firstCol; j <= lastCol; j++) {
				XSSFCell cell = row.getCell(j);
				if (cell == null || cell.getCellType() != XSSFCell.CELL_TYPE_STRING) {
					continue;
				}
				if (!parents.containsKey(j)) {
					continue;
				}
				String text = cell.getStringCellValue();
				if (KUtil.isEmpty(text)) {
					continue;
				}
				if (j == realValueCol) {
					parents.get(j).setRealValue(text);
					continue;
				}
				if (text.startsWith(FLAG_REALVALUE)) {
					parents.get(j).setRealValue(text.substring(FLAG_REALVALUE.length()));
					continue;
				}
				TextBean bean = new TextBean(text);
				// for(int k=j+1;k<=lastCol;k++) {
				// XSSFCell realValueCell = row.getCell(k);
				// if(realValueCell!=null&&realValueCell.getCellType()==XSSFCell.CELL_TYPE_STRING)
				// {
				// String realValue = realValueCell.getStringCellValue();
				// if(!KUtil.isEmpty(realValue)) {
				// bean.setRealValue(realValue);
				// break;
				// }
				// }
				// }
				parents.get(j).children.add(bean);
				parents.put(j + 1, bean);
				// break;
			}
		}
		return root;
	}

	protected List<FileBean> getConfigFile() {
		List<FileBean> files = new ArrayList<>();
		try {
			addFileFromPref(files, PREF_COMMON_FOLDER, "");
		} catch (Exception e) {
			System.out.println("Load Text Tree From <" + PREF_COMMON_FOLDER + "> Error");
			e.printStackTrace();
		}
		try {
			addFileFromPref(files, PREF_PERSONAL_FOLDER, KUtil.session.getUser().getUserId());
		} catch (Exception e) {
			System.out.println("Load Text Tree From <" + PREF_PERSONAL_FOLDER + "> Error");
			e.printStackTrace();
		}
		return files;
	}

	private void addFileFromPref(List<FileBean> files, String prefName, String subFolder) throws Exception {
		String uid = KUtil.getPrefVal(KUtil.session, prefName);
		if (KUtil.isEmpty(uid)) {
			return;
		}
		int realValueColumn = -1;
		int ind = uid.indexOf(PREFFLAG_REALVALUE);
		if (ind > 0) {
			String numStr = uid.substring(ind + PREFFLAG_REALVALUE.length());
			uid = uid.substring(0, ind);
			try {
				realValueColumn = Integer.parseInt(numStr);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		TCComponent comp = KUtil.session.stringToComponent(uid);
		if (comp == null) {
			return;
		}
		if (comp instanceof TCComponentFolder) {
			TCComponentFolder folder = (TCComponentFolder) comp;
			AIFComponentContext[] children = folder.getChildren();
			if (!KUtil.isEmpty(subFolder)) {
				for (AIFComponentContext c : children) {
					InterfaceAIFComponent child = c.getComponent();
					if (child instanceof TCComponentFolder) {
						if (subFolder.equals(((TCComponentFolder) child).getStringProperty("object_name"))) {
							children = child.getChildren();
							break;
						}
					}
				}
			}
			for (AIFComponentContext c : children) {
				InterfaceAIFComponent child = c.getComponent();
				if ("MSExcelX".equals(child.getType())) {
					File[] dbFiles = ((TCComponentDataset) child).getFiles("");
					if (dbFiles != null) {
						for (int i = 0; i < dbFiles.length; i++) {
							files.add(new FileBean(dbFiles[i], realValueColumn));
						}
					}
				}
			}
		} else if (comp instanceof TCComponentDataset) {
			TCComponentDataset dataset = (TCComponentDataset) comp;
			if ("MSExcelX".equals(dataset.getType())) {
				File[] dbFiles = dataset.getFiles("");
				if (dbFiles != null) {
					for (int i = 0; i < dbFiles.length; i++) {
						files.add(new FileBean(dbFiles[i], realValueColumn));
					}
				}
			}
		}
	}

}
