package cn.net.connor.process.newmeop;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.AIFComponentContext;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentBOMLine;
import com.teamcenter.rac.kernel.TCComponentItem;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCComponentItemRevisionType;
import com.teamcenter.rac.kernel.TCComponentItemType;
import com.teamcenter.rac.kernel.TCException;
import com.teamcenter.rac.util.Registry;
import com.teamcenter.services.rac.core.DataManagementService;
import com.teamcenter.services.rac.core._2008_06.DataManagement.CreateIn;
import com.teamcenter.services.rac.core._2008_06.DataManagement.CreateInput;
import com.teamcenter.services.rac.core._2008_06.DataManagement.CreateOut;
import com.teamcenter.services.rac.core._2008_06.DataManagement.CreateResponse;

import cn.net.connor.process.ProcessLocale;
import cn.net.connor.process.ProcessUtil;
import k.KDialogController;
import k.util.KUtil;

public class NewMeopDialogController extends KDialogController {

	private NewMeopDialog dialog;
	private TCComponentBOMLine targetLine;
	private TCComponentItemRevision targetRev;
	public static final String PREF_PREFIX = "Connor_AVIC_MEOP_";
	public static final String PREF_PREFIX_EXCEL = "Connor_AVIC_MEOPEXCEL_";
	public static final String PREF_PROPS = "Connor_AVIC_MEOP_Properties";
	public static final String PREF_REL_LOV = "Connor_AVIC_MEOP_LOV_";
	public static final Registry REG = Registry.getRegistry(NewMeopDialogController.class);

	protected List<MEOPType> meopTypes = new ArrayList<NewMeopDialogController.MEOPType>();
	protected List<PrefComponent> prefComponents = new ArrayList<PrefComponent>();
	protected int propCols = 1;
	protected PrefComponent shortDescPrefComp;

	public class MEOPType {
		protected String type;
		protected String disType;
		protected String additional;
		protected TCComponentItemType itemType;

		public MEOPType(String type, String disType, String additional) throws Exception {
			this.type = type;
			this.disType = disType;
			this.additional = additional;
			this.itemType = (TCComponentItemType) KUtil.session.getTypeComponent(type);
			if (this.itemType == null) {
				throw new Exception(String.format(ProcessLocale.getString("typenotfound.ERROR1"), type));
			}
		}

		@Override
		public String toString() {
			return "  " + disType;
		}
	}

	public NewMeopDialogController(AbstractAIFApplication app) {
		super(app);
	}

	@Override
	public boolean init() throws Exception {
		this.dialog = (NewMeopDialog) aifDialog;
		InterfaceAIFComponent comp = app.getTargetComponent();
		System.out.println("Select: " + comp);
		if (comp != null) {
			if (comp instanceof TCComponentBOMLine) {
				this.targetLine = (TCComponentBOMLine) comp;
				TCComponentItemRevision rev = this.targetLine.getItemRevision();
				System.out.println(">> " + rev.getType());
				if(ProcessUtil.isMEProcess(rev.getItem().getType())) {
				//if (rev instanceof TCComponentMEProcessRevision) {
					this.targetRev = rev;
				}
			}
		}
		if (targetRev == null) {
			KUtil.warn(dialog, ProcessLocale.getString("selectprocessrev.ERROR"));
			return false;
		}
		{
			String prefName = PREF_PREFIX + this.targetRev.getType();
			String[] prefVals = KUtil.getPrefVals(session, prefName);
			int len = KUtil.getLen(prefVals);
			for (int i = 0; i < len; i++) {
				String type = prefVals[i];
				String addition = "";
				int ind = type.indexOf('=');
				if (ind > 0) {
					addition = type.substring(ind + 1).replace("\\n", "\n");
					type = type.substring(0, ind);
				}
				String disType = KUtil.getTypeName(session, type);
				if (KUtil.isEmpty(disType)) {
					System.out.println("Error Config: " + prefVals[i]);
					continue;
				}
				this.meopTypes.add(new MEOPType(type, disType, addition));
			}
			if (this.meopTypes.size() == 0) {
				KUtil.warn(dialog, String.format(ProcessLocale.getString("prefnotconfiged.ERROR1"), prefName));
				return false;
			}
		}
		{
			String[] prefVals = KUtil.getPrefVals(session, PREF_PROPS);
			int len = KUtil.getLen(prefVals);
			for (int i = 0; i < len; i++) {
				String pref = prefVals[i];
				if (pref.length() == 1 && Character.isDigit(pref.charAt(0))) {
					int col = pref.charAt(0) - '0';
					if (col > 0) {
						System.out.println("Reset Proc Col: " + col);
						this.propCols = col;
					}
					continue;
				}
				PrefComponent prefComp = new PrefComponent(pref);
				if (prefComp.init(session, targetRev, prefComponents)) {
					prefComponents.add(prefComp);
				} else {
					System.out.println("Error Config: " + pref);
				}
			}
			if (prefComponents.size() == 0) {
				KUtil.warn(dialog, String.format(ProcessLocale.getString("prefnotconfiged.ERROR1"), PREF_PROPS));
				return false;
			}
			String descPropName = NewMeopDialogController.REG.getString("shortdesc.PROPNAME");
			this.shortDescPrefComp = new PrefComponent(descPropName);
			if(!shortDescPrefComp.init(session, targetRev, prefComponents)) {
				this.shortDescPrefComp = null;
			}
		}

		return true;
	}

	public void assignId() throws Exception {
		MEOPType type = dialog.l_type.getSelectedValue();
		if (type == null) {
			KUtil.warn(dialog, ProcessLocale.getString("selectMeopType.INFO"));
			return;
		}
		TCComponentItemType itemType = type.itemType;
		String newId = itemType.getNewID();
		System.out.println("New ID: " + type.type + " -> " + newId);
		dialog.setAssignId(newId);
	}
	
	public static int getMeopNo(TCComponentBOMLine line) throws Exception{
		TCComponentItemRevision rev = line.getItemRevision();
		if(rev==null) {
			throw new TCException("获取版本失败："+line);
		}
		TCComponent form = rev.getRelatedComponent("IMAN_master_form_rev");
		if(form==null) {
			throw new TCException("获取版本表单失败："+rev);
		}
		form.refresh();
		String seqStr = form.getStringProperty("ac5_OperationNumber");
		System.out.println("Op Number: "+line+" -> "+seqStr);
		int seq = 0;
		if(!KUtil.isEmpty(seqStr)) {
			try {
				seq = Integer.parseInt(seqStr);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return seq;
	}
	
	@SuppressWarnings("deprecation")
	public static void setMeopNo(TCComponentBOMLine line, int meopNo) throws Exception{
		System.out.println("Set Op Number: "+line+" -> "+meopNo);
//		if(meopNo<0) {
//			return;
//		}
		TCComponentItemRevision rev = line.getItemRevision();
		if(rev==null) {
			throw new TCException("获取版本失败："+line);
		}
		TCComponent form = rev.getRelatedComponent("IMAN_master_form_rev");
		if(form==null) {
			throw new TCException("获取版本表单失败："+rev);
		}
		form.refresh();
		form.lock();
		try {
			form.setStringProperty("ac5_OperationNumber", meopNo+"");
			form.save();
			form.refresh();
		} finally {
			form.unlock();
		}
	}
	
	public void assignMeopNo() throws Exception{
		targetLine.refresh();
		AIFComponentContext[] cc = targetLine.getChildren();
		int maxSeq = 0;
		int len = KUtil.getLen(cc);
		for(int i=0;i<len;i++) {
			TCComponentBOMLine cLine = (TCComponentBOMLine) cc[i].getComponent();
			int seq = getMeopNo(cLine);
			maxSeq = Math.max(seq, maxSeq);
		}
		maxSeq+=5;
		System.out.println("New Sequence: "+maxSeq);
//		dialog.setMeopNo(""+maxSeq);
		dialog.setMeopNo(String.format("%02d", maxSeq));
	}

	public void insertMeopNo() throws Exception{
		Map<String, String> propData = new MeopNoDialog(dialog, true, new MeopNoDialogController(app, targetLine)).getMeopInfo();
		System.out.println("Prop Info: "+propData);
		if(propData==null||propData.size()==0) {
			return;
		}
		for (PrefComponent comp : prefComponents) {
			String key = comp.getPropConfig();
			if(propData.containsKey(key)) {
				String value = propData.get(key);
				System.out.println(String.format("set [%s]=[%s]", key, value));
				if(key.equals("revmaster.ac5_OperationNumber") && value.length()==1)
					comp.setValue("0"+value);
				else
					comp.setValue(value);
			}
		}
	}

	public void doCreate() throws Exception{
		System.out.println("doCreate() = = = = = = = =");
		MEOPType typeComp = dialog.l_type.getSelectedValue();
		if (typeComp == null) {
			KUtil.warn(dialog, ProcessLocale.getString("selectMeopType.INFO"));
			return;
		}
		TCComponentItemType itemType = typeComp.itemType;
		//Check Require
		if(!checkRequire()) {
			return;
		}
		String type = itemType.getName();
		System.out.println("Select Type: " + type);
		if (KUtil.isEmpty(type)) {
			KUtil.info(dialog, ProcessLocale.getString("selectType.INFO"));
			return;
		}
		if (!checkRequire()) {
			return;
		}
		TCComponentItemRevisionType revType = itemType.getItemRevisionType();
		String revTypeName = revType.getName();
		String itemMasterTypeName = type+" Master";
		String revMasterTypeName = revTypeName+" Master";
		if(session.getTypeComponent(itemMasterTypeName)==null) {
			itemMasterTypeName = type+"Master";
		}
		if(session.getTypeComponent(revMasterTypeName)==null) {
			revMasterTypeName = revTypeName+"Master";
		}
		System.out.println("Revision Type: " + type);
		System.out.println("Item Master Type: " + itemMasterTypeName);
		System.out.println("Revision Master Type: " + revMasterTypeName);
		Map<String, String> revPropMap = new HashMap<>();
		Map<String, String> revMasterPropMap = new HashMap<>();
		Map<String, String> itemPropMap = new HashMap<>();
		Map<String, String> itemMasterPropMap = new HashMap<>();
		Map<String, String> bomLinePropMap = new HashMap<>();
		for (PrefComponent prefComponent : this.prefComponents) {
			KUtil.initPropMap(prefComponent.propConfig,prefComponent.getValue(),itemPropMap,itemMasterPropMap,revPropMap,revMasterPropMap,bomLinePropMap);
		}
		/*String propType = "ac5_Type";
		//propType = "object_desc";
		if(revMasterPropMap.containsKey(propType)) {
			String ac5Type = revMasterPropMap.get(propType);
			System.out.println(propType+": "+ac5Type);
			if(!KUtil.isEmpty(ac5Type)) {
				revPropMap.put("item_revision_id", ac5Type+"00");
			}
		}*/
		revPropMap.put("item_revision_id", targetLine.getItemRevision().getProperty("item_revision_id"));
		CreateInput revMasterInput = new CreateInput();
		revMasterInput.boName = revMasterTypeName;
		revMasterInput.stringProps = revMasterPropMap;
		CreateInput itemMasterInput = new CreateInput();
		itemMasterInput.boName = itemMasterTypeName;
		itemMasterInput.stringProps = itemMasterPropMap;
		CreateInput revInput = new CreateInput();
		revInput.boName = revTypeName;
		revInput.stringProps = revPropMap;
		Map<String, CreateInput[]> revMasterInfoMap = new HashMap<>();
		revMasterInfoMap.put("IMAN_master_form_rev", new CreateInput[] { revMasterInput });
		revInput.compoundCreateInput = revMasterInfoMap;
		// create rev
		CreateInput itemInput = new CreateInput();
		itemInput.boName = type;
		itemInput.stringProps = itemPropMap;
		Map<String, CreateInput[]> revInfoMap = new HashMap<>();
		revInfoMap.put("revision", new CreateInput[] { revInput });
		revInfoMap.put("IMAN_master_form", new CreateInput[] {itemMasterInput});
		itemInput.compoundCreateInput = revInfoMap;
		// create info
		CreateIn cI = new CreateIn();
		cI.clientId = "CreateItem";
		cI.data = itemInput;
		// service
		DataManagementService service = DataManagementService.getService(session);
		// create
		CreateResponse resp = service.createObjects(new CreateIn[] { cI });
		KUtil.throwServiceDataError(resp.serviceData);
		CreateOut[] cOs = resp.output;
		TCComponentItem newItem = null;
		if (cOs.length > 0) {
			for (TCComponent s : cOs[0].objects) {
				if (s instanceof TCComponentItem) {
					newItem = (TCComponentItem) s;
				}
			}
		}
		System.out.println("New Item: "+newItem);
		if(newItem==null) {
			throw new Exception(ProcessLocale.getString("createprocess.ERROR"));
		}
		newItem.refresh();
		TCComponentItemRevision newRev = newItem.getLatestItemRevision();
		System.out.println("New Rev: "+newRev);
		TCComponentBOMLine newLine = this.targetLine.add(newItem, newRev, null, false);
		if(bomLinePropMap.size()>0) {
			System.out.println("Set Bom Property: "+bomLinePropMap);
			newLine.setProperties(bomLinePropMap);
			newLine.refresh();
		}
		this.targetLine.refresh();
		dialog.setAssignId(null);
		dialog.setMeopNo(null);
		//TODO 
		for(PrefComponent comp : prefComponents) {
			if(!comp.hasDefault())
				comp.setValue("");
		}
	}
	
	private boolean checkRequire() {
		List<String> requireTitles = new ArrayList<String>();
		for (PrefComponent prefComponent : this.prefComponents) {
			if (!prefComponent.checkRequire()) {
				requireTitles.add(prefComponent.title.replace(" ", "").replace(":", "").replace(KUtil.getReg("CN_SYMBEL4"), "").replace(KUtil.getReg("CN_SYMBEL5"), ""));
			}
		}
		if (requireTitles.size() > 0) {
			KUtil.info(dialog, String.format(ProcessLocale.getString("requireprop.INFO1"), requireTitles.toString()));
			return false;
		}
		return true;
	}

	public AbstractAIFApplication getApp() {
		return app;
	}

}
