package cn.net.connor.process.newprocess;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import com.teamcenter.rac.util.MessageBox;

import cn.net.connor.process.ProcessLocale;
import k.KDialog;
import k.KDialogController;
import k.util.KUtil;

@SuppressWarnings("serial")
public class NewProcessDialog extends KDialog {

	private JButton b_ok;
	private JButton b_cel;
	private NewProcessDialogController controller;

	public NewProcessDialog(KDialogController controller) {
		super(controller);
		this.controller = (NewProcessDialogController) controller;
	}

	private void setCompEnabled(boolean b) {
		b_ok.setEnabled(b);
		b_cel.setEnabled(b);
	}

	@Override
	protected void addListeners() {
		this.b_cel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				disposeDialog();
			}
		});
		this.b_ok.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				new Thread(new Runnable() {
					@Override
					public void run() {
						setCompEnabled(false);
						try {
							controller.doCreate();
						} catch (Exception e) {
							e.printStackTrace();
							MessageBox.post(NewProcessDialog.this, e);
						}
//						setCompEnabled(true);
					}
				}).start();
			}
		});
	}

	@Override
	protected void initUI() {
		this.setTitle(ProcessLocale.getString("newprocess.TITLE"));
		this.setPreferredSize(new Dimension(640, 640));
		this.setMinimumSize(new Dimension(640, 480));
		this.setLayout(new BorderLayout());
		JSplitPane pane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		pane.setTopComponent(initProcPanel());
		pane.setBottomComponent(initItemPanel());
		pane.setBorder(null);
		this.add(BorderLayout.CENTER, pane);
		this.add(BorderLayout.SOUTH, initBtnPanel());
	}

	private Component initBtnPanel() {
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER, 20, 5));
		this.b_ok = new JButton(ProcessLocale.getString("ok"));
		this.b_cel = new JButton(ProcessLocale.getString("cancel"));
		panel.add(b_ok);
		panel.add(b_cel);
		return panel;
	}

	private Component initProcPanel() {
		String title = ProcessLocale.getString("procgroup.TITLE");
		List<PrefComponent> comps = controller.procComps;
		int colCnt = controller.procCols;
		JPanel wrapper = new JPanel(new BorderLayout());
		wrapper.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(30,5,5,5), title));
		JPanel panel = new JPanel(new GridBagLayout());
		GridBagConstraints s = getGridBagConstraints(0, 0, new Insets(4, 0, 4, 25));
		int size = comps.size();
		for(int i=0;i<size;i++) {
			PrefComponent prefComponent = comps.get(i);
			if(i%colCnt==0) {
				KUtil.newRow(panel, prefComponent.getPanel(), s, 1);
			}else {
				KUtil.addCol(panel, prefComponent.getPanel(), s, 1, 0);
			}
		}
		s.weighty = 1;
		KUtil.newRow(panel, new JLabel(""), s, 1);
		JScrollPane scroll = new JScrollPane(panel);
		scroll.setBorder(null);
		wrapper.add(BorderLayout.CENTER, scroll);
		return wrapper;
	}

	private Component initItemPanel() {
		String title = ProcessLocale.getString("itemgroup.TITLE");
		List<PrefComponent> comps = controller.itemComps;
		int colCnt = controller.itemCols;
		JPanel wrapper = new JPanel(new BorderLayout());
		wrapper.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(30,5,5,5), title));
		JPanel panel = new JPanel(new GridBagLayout());
		GridBagConstraints s = getGridBagConstraints(0, 0, new Insets(4, 0, 4, 25));
		int size = comps.size();
		for(int i=0;i<size;i++) {
			PrefComponent prefComponent = comps.get(i);
			if(i%colCnt==0) {
				KUtil.newRow(panel, prefComponent.getPanel(), s, 1);
			}else {
				KUtil.addCol(panel, prefComponent.getPanel(), s, 1, 0);
			}
		}
		s.weighty = 1;
		KUtil.newRow(panel, new JLabel(""), s, 1);
		JScrollPane scroll = new JScrollPane(panel);
		scroll.setBorder(null);
		wrapper.add(BorderLayout.CENTER, scroll);
		return wrapper;
	}

}
