package cn.net.connor.process.newprocess;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentFolder;
import com.teamcenter.rac.kernel.TCComponentItem;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCComponentItemRevisionType;
import com.teamcenter.rac.kernel.TCComponentItemType;
import com.teamcenter.rac.util.MessageBox;
import com.teamcenter.services.rac.core.DataManagementService;
import com.teamcenter.services.rac.core._2008_06.DataManagement.CreateIn;
import com.teamcenter.services.rac.core._2008_06.DataManagement.CreateInput;
import com.teamcenter.services.rac.core._2008_06.DataManagement.CreateOut;
import com.teamcenter.services.rac.core._2008_06.DataManagement.CreateResponse;

import cn.net.connor.process.ProcessLocale;
import k.KDialogController;
import k.util.KUtil;

public class NewProcessDialogController extends KDialogController {

	private NewProcessDialog dialog;
	private TCComponentItemRevision targetRev;
	public static final String PREF_PREFIX = "Connor_AVIC_";
	public static final String PREF_GROUP_SPLITTER = "---";
	public static final String PROP_OBJECT_TYPE = "object_type";
	protected List<PrefComponent> procComps = new ArrayList<PrefComponent>();
	protected List<PrefComponent> itemComps = new ArrayList<PrefComponent>();
	private PrefComponent objectTypeComp;
	protected int procCols = 2;
	protected int itemCols = 2;
	public static final String PREF_PROCTYPE_SYMBOL = "Connor_AVIC_ProcType_Symbol";
	private Map<String, String> procTypeSymbolMap = new HashMap<String, String>();
	private Map<PrefComponent, String> symbolCompMap = new HashMap<PrefComponent, String>();
	public static final String FLAG_TYPE_SYMBOL = "[PROC_TYPE]";

	public NewProcessDialogController(AbstractAIFApplication app) {
		super(app);
	}

	@Override
	public boolean init() throws Exception {
		this.dialog = (NewProcessDialog) aifDialog;
		InterfaceAIFComponent comp = app.getTargetComponent();
		if (comp == null || !(comp instanceof TCComponentItemRevision)) {
			KUtil.warn(dialog, ProcessLocale.getString("selectnewprocesstarget.ERROR"));
			return false;
		}
		this.targetRev = (TCComponentItemRevision) comp;
		String revType = this.targetRev.getType();
		System.out.println("Select Rev: " + this.targetRev + "| " + revType);
		String prefName = PREF_PREFIX + revType;
		System.out.println("Load Preference: " + prefName);
		String[] prefVals = KUtil.getPrefVals(session, prefName);
		int len = KUtil.getLen(prefVals);
		if (len == 0) {
			System.out.println(String.format(ProcessLocale.getString("prefnotconfiged.ERROR1"), prefName));
			KUtil.warn(dialog, ProcessLocale.getString("selectnewprocesstarget.ERROR"));
			return false;
		}
		List<PrefComponent> compList = procComps;
		for (int i = 0; i < len; i++) {
			String pref = prefVals[i];
			if (pref.length() == 1 && Character.isDigit(pref.charAt(0))) {
				if (compList == itemComps) {
					int col = pref.charAt(0) - '0';
					if (col > 0) {
						System.out.println("Reset Item Col: " + col);
						itemCols = col;
					}
				} else if (compList == procComps) {
					int col = pref.charAt(0) - '0';
					if (col > 0) {
						System.out.println("Reset Proc Col: " + col);
						procCols = col;
					}
				}
				continue;
			}
			if (pref.startsWith(PREF_GROUP_SPLITTER)) {
				compList = itemComps;
				continue;
			}
			PrefComponent prefComp = new PrefComponent(pref, this.targetRev);
			if (prefComp.init(session)) {
				if (PROP_OBJECT_TYPE.equals(prefComp.propConfig)) {
					this.objectTypeComp = prefComp;
				}
				compList.add(prefComp);
			} else {
				System.out.println("Error Config: " + pref);
			}
		}
		if (procComps.size() + itemComps.size() == 0) {
			KUtil.warn(dialog, String.format(ProcessLocale.getString("prefnotconfiged.ERROR1"), prefName));
			return false;
		}
		for (PrefComponent c : procComps) {
			if (c.textField != null && c.getValue() != null && c.getValue().contains(FLAG_TYPE_SYMBOL)) {
				this.symbolCompMap.put(c, c.getValue());
				c.textField.setText(c.getValue().replace(FLAG_TYPE_SYMBOL, ""));
			}
		}
		for (PrefComponent c : itemComps) {
			if (c.textField != null && c.getValue() != null && c.getValue().contains(FLAG_TYPE_SYMBOL)) {
				this.symbolCompMap.put(c, c.getValue());
				c.textField.setText(c.getValue().replace(FLAG_TYPE_SYMBOL, ""));
			}
		}
		{
			String[] prefVals2 = KUtil.getPrefVals(session, PREF_PROCTYPE_SYMBOL);
			int len2 = KUtil.getLen(prefVals2);
			for (int i = 0; i < len2; i++) {
				String pref = prefVals2[i];
				int ind = pref.indexOf('=');
				if (ind > 0) {
					String type = pref.substring(0, ind);
					String symbol = pref.substring(ind + 1);
					System.out.println("Type Symbol: " + type + " -> " + symbol);
					this.procTypeSymbolMap.put(type, symbol);
				}
			}
		}
		if (this.symbolCompMap.size() > 0 && this.objectTypeComp != null && this.objectTypeComp.comboBox != null) {
			this.objectTypeComp.comboBox.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					String type = objectTypeComp.getValue();
					String symbol = procTypeSymbolMap.containsKey(type) ? procTypeSymbolMap.get(type) : "";
					System.out.println("Select Type: " + type + " -> " + symbol);
					for (PrefComponent c : symbolCompMap.keySet()) {
						String text = symbolCompMap.get(c);
						System.out.println(text);
						text = text.replace(FLAG_TYPE_SYMBOL, symbol);
						c.textField.setText(text);
					}

				}
			});
		}
		return true;
	}

	public void doCreate() throws Exception {
		if (this.objectTypeComp == null) {
			throw new Exception(ProcessLocale.getString("typecompnotfound.ERROR"));
		}
		String type = this.objectTypeComp.getValue();
		String relName = this.objectTypeComp.getAddtionalValue();
		System.out.println("Select Type: " + type);
		if (KUtil.isEmpty(type)) {
			KUtil.info(dialog, ProcessLocale.getString("selectType.INFO"));
			return;
		}
		TCComponentItemType itemType = (TCComponentItemType) session.getTypeComponent(type);
		if (itemType == null) {
			throw new Exception(String.format(ProcessLocale.getString("typenotfound.ERROR1"), type));
		}
		if (!checkRequire()) {
			return;
		}
		boolean addedToNewRev;
		try {
			KUtil.setByPass(true);
			TCComponentItemRevisionType revType = itemType.getItemRevisionType();
			String revTypeName = revType.getName();
			String itemMasterTypeName = type + " Master";
			String revMasterTypeName = revTypeName + " Master";
			if (session.getTypeComponent(itemMasterTypeName) == null) {
				itemMasterTypeName = type + "Master";
			}
			if (session.getTypeComponent(revMasterTypeName) == null) {
				revMasterTypeName = revTypeName + "Master";
			}
			System.out.println("Revision Type: " + type);
			System.out.println("Item Master Type: " + itemMasterTypeName);
			System.out.println("Revision Master Type: " + revMasterTypeName);
			Map<String, String> revPropMap = new HashMap<>();
			Map<String, String> revMasterPropMap = new HashMap<>();
			Map<String, String> itemPropMap = new HashMap<>();
			Map<String, String> itemMasterPropMap = new HashMap<>();
			for (PrefComponent prefComponent : itemComps) {
				KUtil.initPropMap(prefComponent.propConfig, prefComponent.getValue(), itemPropMap, itemMasterPropMap, revPropMap, revMasterPropMap);
			}
			for (PrefComponent prefComponent : procComps) {
				KUtil.initPropMap(prefComponent.propConfig, prefComponent.getValue(), itemPropMap, itemMasterPropMap, revPropMap, revMasterPropMap);
			}
			String name = itemPropMap.get("object_name");
			Pattern pattern = Pattern.compile("[\\\\/:*?\\\"<>|]");
			Matcher matcher = pattern.matcher(name);
			if(matcher.find()) {//TODO \/:*?鈥溾��<>|
				throw new Exception(ProcessLocale.getString("name.ERROR"));
			}
			CreateInput revMasterInput = new CreateInput();
			revMasterInput.boName = revMasterTypeName;
			revMasterInput.stringProps = revMasterPropMap;
			CreateInput itemMasterInput = new CreateInput();
			itemMasterInput.boName = itemMasterTypeName;
			itemMasterInput.stringProps = itemMasterPropMap;
			CreateInput revInput = new CreateInput();
			revInput.boName = revTypeName;
			revInput.stringProps = revPropMap;
			Map<String, CreateInput[]> revMasterInfoMap = new HashMap<>();
			revMasterInfoMap.put("IMAN_master_form_rev", new CreateInput[] { revMasterInput });
			revInput.compoundCreateInput = revMasterInfoMap;
			// create rev
			CreateInput itemInput = new CreateInput();
			itemInput.boName = type;
			itemInput.stringProps = itemPropMap;
			Map<String, CreateInput[]> revInfoMap = new HashMap<>();
			revInfoMap.put("revision", new CreateInput[] { revInput });
			revInfoMap.put("IMAN_master_form", new CreateInput[] { itemMasterInput });
			itemInput.compoundCreateInput = revInfoMap;
			// create info
			CreateIn cI = new CreateIn();
			cI.clientId = "CreateItem";
			cI.data = itemInput;
			// service
			DataManagementService service = DataManagementService.getService(session);
			// create
			CreateResponse resp = service.createObjects(new CreateIn[] { cI });
			KUtil.throwServiceDataError(resp.serviceData);
			CreateOut[] cOs = resp.output;
			TCComponentItem newItem = null;
			if (cOs.length > 0) {
				for (TCComponent s : cOs[0].objects) {
					if (s instanceof TCComponentItem) {
						newItem = (TCComponentItem) s;
					}
				}
			}
			System.out.println("New Item: " + newItem);
			if (newItem == null) {
				throw new Exception(ProcessLocale.getString("createprocess.ERROR"));
			}
			newItem.refresh();
			TCComponentItemRevision newRev = newItem.getLatestItemRevision();
			System.out.println("New Rev: " + newRev);
			System.out.println("Create Relation: " + relName);
			addedToNewRev = false;
			if (newRev != null && !KUtil.isEmpty(relName)) {
				try {
					String archiveRelation = null;
					int ind = relName.indexOf('-');
					if(ind>0) {
						archiveRelation = relName.substring(ind+1);
						relName = relName .substring(0, ind);
					}
					newRev.add(relName, this.targetRev);
					newRev.refresh();
					if(!KUtil.isEmpty(archiveRelation)) {
						this.targetRev.add(archiveRelation, newItem);
						this.targetRev.refresh();
						addedToNewRev = true;
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			if (!addedToNewRev) {
				TCComponentFolder folder = session.getUser().getNewStuffFolder();
				folder.add("contents", newItem);
				folder.refresh();
				
			} 
		} finally {
			KUtil.setByPass(false);
		}
		dialog.dispose();
		MessageBox.post(addedToNewRev ? ProcessLocale.getString("newprocesscomplete.INFO") : ProcessLocale.getString("newprocesscomplete.ERROR"), "提示", MessageBox.INFORMATION);
		//KUtil.info(dialog, addedToNewRev ? ProcessLocale.getString("newprocesscomplete.INFO") : ProcessLocale.getString("newprocesscomplete.ERROR"));
		
	}

	private boolean checkRequire() {
		List<String> requireTitles = new ArrayList<String>();
		for (PrefComponent prefComponent : itemComps) {
			if (!prefComponent.checkRequire()) {
				requireTitles.add(prefComponent.title.replace(" ", "").replace(":", "").replace(KUtil.getReg("CN_SYMBEL4"), "").replace(KUtil.getReg("CN_SYMBEL5"), ""));
			}
		}
		for (PrefComponent prefComponent : procComps) {
			if (!prefComponent.checkRequire()) {
				requireTitles.add(prefComponent.title.replace(" ", "").replace(":", "").replace(KUtil.getReg("CN_SYMBEL4"), "").replace(KUtil.getReg("CN_SYMBEL5"), ""));
			}
		}
		if (requireTitles.size() > 0) {
			KUtil.info(dialog, String.format(ProcessLocale.getString("requireprop.INFO1"), requireTitles.toString()));
			return false;
		}
		return true;
	}

}
