package cn.net.connor.process.newprocess;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JPanel;

import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCProperty;
import com.teamcenter.rac.kernel.TCSession;
import com.teamcenter.rac.util.iTextField;
import com.teamcenter.rac.util.combobox.iComboBox;

import k.KLocale;
import k.util.KUtil;

public class PrefComponent {

	public static final String REQUIRE = "*";
	public static final String READONLY = "#";

	private JPanel panel;
	private String configStr;
	protected boolean require = false;
	protected boolean readonly = false;
	protected String title;
	protected String propConfig;
	protected String fromPropConfig;
	private String[] additional;
	private String[] lov;
	private String[] displayLov;
	protected iTextField textField;
	protected iComboBox comboBox;
	private int textLength = -1;
	private TCComponentItemRevision rev;

	public PrefComponent(String pref, TCComponentItemRevision targetRev) {
		this.configStr = pref;
		this.rev = targetRev;
	}

	public boolean init(TCSession session) throws Exception{
		if (KUtil.isEmpty(configStr)) {
			return false;
		}
		if (configStr.startsWith(REQUIRE)) {
			configStr = configStr.substring(1);
			this.require = true;
		}
		if (configStr.startsWith(READONLY)) {
			configStr = configStr.substring(1);
			this.readonly = true;
		}
		String[] split = configStr.split("=", -1);
		int len = KUtil.getLen(split);
		if (len > 0) {
			this.title = split[0];
		}
		if (len > 1) {
			this.propConfig = split[1];
		}
		if (len > 2) {
			this.fromPropConfig = split[2];
		}
		if (len > 3 && !KUtil.isEmpty(split[3])) {
			this.lov = split[3].split(";");
		}
		if (len > 4 && !KUtil.isEmpty(split[4])) {
			try {
				this.textLength = Integer.parseInt(split[4].trim());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (KUtil.isEmpty(title)) {
			return false;
		}
		if ("object_type".equals(propConfig)) {
			int lovCnt = KUtil.getLen(lov);
			displayLov = new String[lovCnt];
			additional = new String[lovCnt];
			for (int i = 0; i < lovCnt; i++) {
				String typeName = lov[i];
				String relName = "";
				int ind = typeName.indexOf('-');
				if(ind>0) {
					relName = typeName.substring(ind+1);
					typeName = typeName.substring(0,ind);
				}
				lov[i]=typeName;
				displayLov[i] = KUtil.getTypeName(session, typeName);
				additional[i] = relName;
			}
		} else {
			displayLov = lov;
		}
		initPanel();
		initComponent();
		return true;
	}

	private void initComponent() throws Exception{
		if(readonly) {
			if(textField!=null) {
				textField.setEditable(false);
			}
			if(comboBox!=null) {
				comboBox.setEditable(false);
				comboBox.setEnabled(false);
			}
		}
		if(require) {
			if(textField!=null) {
				textField.setRequired(true);
			}
			if(comboBox!=null) {
				comboBox.setMandatory(true);
			}
		}
		if(rev!=null&&!KUtil.isEmpty(fromPropConfig)) {
			String value = getConfigPropValue(rev,fromPropConfig);
			System.out.println(">> Load Property: "+fromPropConfig+" = "+value);
			if(textField!=null) {
				textField.setText(value);
			}
			if(comboBox!=null) {
				comboBox.setSelectedItem(value);
			}
		}
	}
	
	public static String getConfigPropValue(TCComponentItemRevision rev, String propConfigStr) throws Exception {
		if (rev == null || KUtil.isEmpty(propConfigStr)) {
			return "";
		}
		StringBuilder res = new StringBuilder();
		String[] propConfigSplit = propConfigStr.split("\\+");
		int len = KUtil.getLen(propConfigSplit);
		for (int i = 0; i < len; i++) {
			String propConfig = propConfigSplit[i];
			String propFrom = null, propName = null;
			String[] split = propConfig.split("\\.");
			if (KUtil.getLen(split) == 2) {
				propFrom = split[0].trim().toUpperCase();
				propName = split[1].trim();
			}
			if (KUtil.isEmpty(propFrom) || KUtil.isEmpty(propName)) {
				res.append(propConfig);
				continue;
			}
			TCComponent target = null;
			if ("ITEM".equals(propFrom)) {
				target = rev.getItem();
			} else if ("REV".equals(propFrom)) {
				target = rev;
			} else if ("IRELA".equals(propFrom)) {
				TCComponent[] targets = rev.getRelatedComponents("Ac5_Processroute_Irela");
				int targetCnt = KUtil.getLen(targets);
				for(int k=0;k<targetCnt;k++) {
					if("Ac5_Processroute_FM".equals(targets[k].getType())) {
						target = targets[k];
						break;
					}
				}
			} else if ("ITEMMASTER".equals(propFrom)) {
				target = rev.getItem().getRelatedComponent("IMAN_master_form");
			} else if ("REVMASTER".equals(propFrom)) {
				target = rev.getRelatedComponent("IMAN_master_form_rev");
			} else {
				throw new Exception(String.format(KLocale.getString("proplocationnotdefined.ERROR1"), propFrom));
			}
			if(target !=null) {
				TCProperty prop = target.getTCProperty(propName.trim());
				if (prop == null) {
					throw new Exception(String.format(KLocale.getString("propnotdefinedintype.ERROR2"), target.getType(), propName));
				}
				res.append(prop.getDisplayableValue());
			}
		}
		return res.toString();
	}

	private void initPanel() {
		this.panel = new JPanel(new GridBagLayout());
		this.panel.setPreferredSize(new Dimension(260, 40));
		GridBagConstraints s = new GridBagConstraints(0, 0, 1, 1, 0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 0, 0), 0, 0);
		JLabel label = new JLabel(title);
		//label.setMinimumSize(new Dimension(120, 0));
		KUtil.newRow(panel, label, s, 1);
		int len = KUtil.getLen(displayLov);
		if (len > 0) {
			comboBox = new iComboBox();
			for (int i = 0; i < len; i++) {
				comboBox.addItem(displayLov[i]);
			}
			KUtil.addCol(panel, comboBox, s, 1, 1);
		} else {
			textField = new iTextField(textLength > 0 ? textLength : 12);
			KUtil.addCol(panel, textField, s, 1, textLength > 0 ? 0 : 1);
			if(textLength>0) {
				KUtil.addCol(panel, new JLabel(""), s, 1, 1);
			}
		}
	}

	public Component getPanel() {
		return panel;
	}
	
	public String getValue() {
		String value = null;
		if(textField!=null) {
			value = textField.getText();
		}else if(comboBox!=null) {
			int ind = comboBox.getSelectedIndex();
			if(ind>=0&&ind<KUtil.getLen(lov)) {
				value = lov[ind];
			}
		}
		return value;
	}
	
	public String getAddtionalValue() {
		String value = null;
		if(textField!=null) {
			value = textField.getText();
		}else if(comboBox!=null) {
			int ind = comboBox.getSelectedIndex();
			if(ind>=0&&ind<KUtil.getLen(lov)) {
				value = additional[ind];
			}
		}
		return value;
	}

	public boolean checkRequire() {
		if(!require) {
			return true;
		}
		String val = getValue();
		return val!=null&&!"".equals(val);
	}

}
