package cn.net.connor.process.openpcb;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.swing.filechooser.FileSystemView;

import org.apache.commons.compress.archivers.sevenz.SevenZArchiveEntry;
import org.apache.commons.compress.archivers.sevenz.SevenZFile;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.kernel.TCComponentDataset;
import com.teamcenter.rac.kernel.TCException;

import cn.net.connor.process.ProcessLocale;
import k.KOperation;
import k.util.KUtil;
import net.lingala.zip4j.core.ZipFile;

public class OpenPCBOperation extends KOperation {

	private TCComponentDataset dataset;

	public OpenPCBOperation(AbstractAIFApplication app, String opName) {
		super(app, opName);
	}

	@Override
	public boolean init() throws Exception {
		InterfaceAIFComponent target = app.getTargetComponent();
		String type = target == null ? null : target.getType();
		System.out.println("Select: " + target + ", " + type);
		if (!"Zip".equals(type)) {
			throw new TCException(ProcessLocale.getString("selectpcb.INFO"));
		}
		this.dataset = (TCComponentDataset) target;
		String name = dataset.getStringProperty("object_name");
		System.out.println("Name: " + name);
		if (!name.toUpperCase().contains(".PRJPCB")) {
			throw new TCException(ProcessLocale.getString("selectpcb.INFO"));
		}
//		if (!name.contains(".PrjPCB")) {
//			throw new TCException(ProcessLocale.getString("selectpcb.INFO"));
//		}
		return true;
	}

	public static void decompress(File file, File destination, String ext, List<File> targetFiles) throws Exception {
		SevenZFile sevenZFile = new SevenZFile(file);
		SevenZArchiveEntry entry;
		while ((entry = sevenZFile.getNextEntry()) != null) {
			if (entry.isDirectory()) {
				continue;
			}
			File curfile = new File(destination, entry.getName());
			File parent = curfile.getParentFile();
			if (!parent.exists()) {
				parent.mkdirs();
			}
			if (curfile.getAbsolutePath().endsWith(ext)) {
				targetFiles.add(curfile);
			}
			FileOutputStream out = new FileOutputStream(curfile);
			byte[] content = new byte[(int) entry.getSize()];
			sevenZFile.read(content, 0, content.length);
			out.write(content);
			out.close();
		}
		sevenZFile.close();
	}

	public void extract7z(File file) throws Exception {
		FileSystemView fsv = FileSystemView.getFileSystemView();
		String deskPath = fsv.getHomeDirectory().getPath();
		File extractPath = new File(deskPath + File.separator + file.getName().replace(".zip", "").replace(".ZIP", ""));
		if (!extractPath.isDirectory()) {
			extractPath.mkdirs();
		}
		System.out.println("Extract to: " + extractPath);
		List<File> pcbFiles = new ArrayList<File>();
		decompress(file, extractPath, ".PrjPCB", pcbFiles);
		System.out.println("Find PCB File: " + pcbFiles);
		if (pcbFiles.size() == 0) {
			throw new TCException(ProcessLocale.getString("pcbfilenotfound.ERROR"));
		}
		for (File f : pcbFiles) {
			KUtil.openFile(f);
		}
	}

	@Override
	public void execute() throws Exception {
		File file = KUtil.getDatasetFile(dataset);
		System.out.println("Download File: " + file.getAbsolutePath());
		ZipFile zipFile = new ZipFile(file);
		if (!zipFile.isValidZipFile()) {
			System.out.println("Not Valid Zip File, Try 7z");
			extract7z(file);
			return;
		}
		zipFile.setFileNameCharset("GBK");
		FileSystemView fsv = FileSystemView.getFileSystemView();
		String deskPath = fsv.getHomeDirectory().getPath();
		System.out.println("Extract to: " + deskPath);
		zipFile.extractAll(deskPath);
		String name = file.getName().replace(".zip", "").replace(".ZIP", "");
		String extractFolder = deskPath + File.separator + name;
		System.out.println("Check Folder: " + extractFolder);
		File folder = new File(extractFolder);
		if (!folder.isDirectory()) {
			throw new TCException(String.format(ProcessLocale.getString("extractpathnotfound.ERROR1"), extractFolder));
		}
		List<File> pcbFiles = new ArrayList<File>();
		File[] files = folder.listFiles();
		int len = KUtil.getLen(files);
		for (int i = 0; i < len; i++) {
			File f = files[i];
			if (f.isDirectory()) {
				continue;
			}
			System.out.println(">> " + f.getAbsolutePath());
			if ((f.getAbsolutePath().toUpperCase()).endsWith(".PRJPCB")) {
				pcbFiles.add(f);
			}
//			if (f.getAbsolutePath().endsWith(".PrjPCB")) {
//				pcbFiles.add(f);
//			}
		}
		System.out.println("Find PCB File: " + pcbFiles);
		if (pcbFiles.size() == 0) {
			throw new TCException(ProcessLocale.getString("pcbfilenotfound.ERROR"));
		}
		for (File f : pcbFiles) {
			KUtil.openFile(f);
		}
	}

}
