package cn.net.connor.process.openpcb;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentDataset;
import com.teamcenter.rac.kernel.TCComponentItem;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCException;

import cn.net.connor.process.ProcessLocale;
import k.KOperation;
import k.util.KUtil;

public class OpenPCBOperation2 extends KOperation {

	private TCComponentItemRevision projRev;
	private File parentFolder;
	public static final String PREF_PCB = "Connor_AVIC_OpenInAD";
	private List<PCBConfig> pcbConfigs = new ArrayList<PCBConfig>();
	private String[] exts;
	private String folderNameConfig;
	private String projVersion;
	public static final String FLAG_OPEN = "OPEN=";
	public static final String FLAG_NAME = "FOLDER=";
	private int fileCounter = 0;

	public OpenPCBOperation2(AbstractAIFApplication app, String opName) {
		super(app, opName);
	}

	@Override
	public boolean init() throws Exception {
		InterfaceAIFComponent target = app.getTargetComponent();
		String type = target == null ? null : target.getType();
		System.out.println("Select: " + target + ", " + type);
		if (target == null || !(target instanceof TCComponentItemRevision)) {
			throw new TCException(ProcessLocale.getString("selectpcbrev.INFO"));
		}
		this.projRev = (TCComponentItemRevision) target;
		this.projVersion = this.projRev.getStringProperty("item_revision_id");
		String[] prefVals = KUtil.getPrefVals(session, PREF_PCB);
		int len = KUtil.getLen(prefVals);
		if (len < 2) {
			throw new TCException(String.format(ProcessLocale.getString("preferror.ERROR1"), PREF_PCB));
		}
		String folderPath = prefVals[0];
		System.out.println("Download to folder: " + folderPath);
		this.parentFolder = new File(folderPath);
		if (!parentFolder.isDirectory()) {
			if (!parentFolder.mkdirs()) {
				throw new TCException(String.format(ProcessLocale.getString("mkdir.ERROR1"), folderPath));
			}
		}
		for (int i = 1; i < len; i++) {
			if (prefVals[i].startsWith(FLAG_OPEN)) {
				this.exts = prefVals[i].substring(FLAG_OPEN.length()).split(":");
				System.out.println(">> Open ext: " + Arrays.toString(exts));
				continue;
			}
			if (prefVals[i].startsWith(FLAG_NAME)) {
				this.folderNameConfig = prefVals[i].substring(FLAG_NAME.length());
				System.out.println(">> Folder Name: " + folderNameConfig);
				continue;
			}
			String[] config = prefVals[i].split(":", -1);
			if (KUtil.getLen(config) == 4) {
				String rel_proj_item = config[0];
				String type_item = config[1];
				String rel_rev_dataset = config[2];
				String suffix_dataset = config[3];
				System.out.println(">> Add Config: " + prefVals[i]);
				PCBConfig pcbConfig = new PCBConfig(rel_proj_item, type_item, rel_rev_dataset, suffix_dataset);
				pcbConfigs.add(pcbConfig);
			} else {
				System.out.println(">> Error Pref Value: " + prefVals[i]);
			}
		}
		if (this.pcbConfigs.size() == 0) {
			throw new TCException(String.format(ProcessLocale.getString("preferror.ERROR1"), PREF_PCB));
		}
		return true;
	}
	
	private TCComponentItemRevision getRev(TCComponent item) throws Exception{
		if(item==null||!(item instanceof TCComponentItem)) {
			return null;
		}
		TCComponent[] revs = item.getReferenceListProperty("revision_list");
		System.out.println("Revs: "+Arrays.toString(revs));
		int len = KUtil.getLen(revs);
		TCComponent res = null;
		for(int i=0;i<len;i++) {
			TCComponent rev = revs[i];
			if(projVersion.equals(rev.getStringProperty("item_revision_id"))) {
				if(res==null) {
					res = rev;
				}else {
					int resSequenceId = res.getIntProperty("sequence_id");
					int tempSequenceId = rev.getIntProperty("sequence_id");
					if (resSequenceId < tempSequenceId) {
						res = rev;
						System.out.println("Find Later Sequence" + rev + "|" + tempSequenceId);
					}
				}
			}
		}
		return (TCComponentItemRevision) res;
	}

	private void downloadFiles(File dir, PCBConfig config, List<File> filesToOpen) throws Exception {
		System.out.println("Get File: "+config);
		List<TCComponent> items = KUtil.getChildComponents(projRev, config.type_item, config.rel_proj_item);
		System.out.println(">> Find Items: "+items);
		int itemCnt = items==null?0:items.size();
		for (int c = 0;c<itemCnt;c++) {
			TCComponent item = items.get(c);
			System.out.println((c+1)+". Check Item: "+item);
			TCComponentItemRevision rev = getRev(item);
			System.out.println("Find Revision: "+rev);
			if(rev==null) {
				continue;
			}
			List<TCComponentDataset> datasets = getDatasets(rev, config.rel_rev_dataset, config.suffix_dataset);
			System.out.println("Find Datasets: "+datasets);
			int size= datasets==null?0:datasets.size();
			for(int i=0;i<size;i++) {
				TCComponentDataset dataset = datasets.get(i);
				File[] files = dataset.getFiles("", dir.getAbsolutePath());
				System.out.println("Download Files: "+Arrays.toString(files));
				checkOpen(files, filesToOpen);
			}
		}
	}

	private void checkOpen(File[] files, List<File> filesToOpen) {
		int fileCnt  = KUtil.getLen(files);
		if(fileCnt==0) {
			return;
		}
		fileCounter+=fileCnt;
		int extCnt = KUtil.getLen(exts);
		if(extCnt==0) {
			return;
		}
		for(int j=0;j<fileCnt;j++) {
			String filePath = files[j].getAbsolutePath();
			int ind = filePath.lastIndexOf('.');
			String ext = filePath.substring(ind+1);
			if(!KUtil.isEmpty(ext)&&KUtil.inArray(ext, exts)>=0) {
				filesToOpen.add(files[j]);
			}
		}
	}

	private List<TCComponentDataset> getDatasets(TCComponent rev, String rel, String dsNamePrefix) throws Exception {
		if (rev == null) {
			return null;
		}
		rev.refresh();
		List<TCComponentDataset> res = new ArrayList<TCComponentDataset>();
		TCComponent[] kids = rev.getRelatedComponents(rel);
		for (TCComponent c : kids) {
			if ((c instanceof TCComponentDataset) && c.getProperty("object_name").endsWith(dsNamePrefix)) {
				res.add((TCComponentDataset) c);
			}
		}
		return res;
	}

	@Override
	public void execute() throws Exception {
		session.setStatus(ProcessLocale.getString("initfolder.STATUS"));
		// get folder
		File dir = this.parentFolder;
		if (!KUtil.isEmpty(folderNameConfig)) {
			int ind = folderNameConfig.indexOf(':');
			String folderName = null;
			if (ind > 0) {
				String rel_proj_dataset = folderNameConfig.substring(0, ind);
				String dsSuffix = folderNameConfig.substring(ind + 1);
				List<TCComponentDataset> datasets = getDatasets(projRev, rel_proj_dataset, dsSuffix);
				System.out.println("Find Proj Datasets: " + datasets);
				if (datasets.size() > 0) {
					folderName = datasets.get(0).getProperty("object_name");
				}
			} else {
				folderName = folderNameConfig;
			}
			System.out.println("Target Folder: " + folderName);
			if (!KUtil.isEmpty(folderName)) {
				dir = new File(this.parentFolder.getAbsolutePath() + File.separator + folderName);
				if (!dir.isDirectory()) {
					if (!dir.mkdirs()) {
						throw new TCException(String.format(ProcessLocale.getString("mkdir.ERROR1"), dir.getAbsolutePath()));
					}
				}
			} else {
				throw new TCException(ProcessLocale.getString("getfoldername.ERROR"));
			}
		}
		// clear folder
		String[] entries = dir.list();
		for (String s : entries) {
			File currentFile = new File(dir.getPath(), s);
			if (!currentFile.delete()) {
				throw new TCException(String.format(ProcessLocale.getString("deletefile.ERROR1"), currentFile.getAbsolutePath()));
			}
		}
		// download file
		ProcessLocale.getString("downloadfile.STATUS");
		List<File> filesToOpen = new ArrayList<File>();
		int configCnt = this.pcbConfigs.size();
		for (int i = 0; i < configCnt; i++) {
			PCBConfig pcbConfig = pcbConfigs.get(i);
			downloadFiles(dir, pcbConfig, filesToOpen);
		}
		if(fileCounter==0) {
			throw new TCException(ProcessLocale.getString("nodataset.ERROR"));
		}
		// open files
		ProcessLocale.getString("openfile.status");
		System.out.println("Open Files: "+filesToOpen);
		for (File f : filesToOpen) {
			KUtil.openFile(f);
		}
	}

}
