package cn.net.connor.process.openpcb;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.io.FileUtils;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentDataset;
import com.teamcenter.rac.kernel.TCComponentItem;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCException;

import cn.net.connor.process.ProcessLocale;
import k.KOperation;
import k.util.KUtil;

public class OpenPCBOperation3 extends KOperation {

	private TCComponentItemRevision projRev;
	private File parentFolder;
	public static final String PREF_PCB = "Connor_AVIC_OpenInAD";
	private List<PCBConfig> pcbConfigs = new ArrayList<PCBConfig>();
	private String[] exts;
	private String folderNameConfig;
	private String projVersion;
	public static final String FLAG_OPEN = "OPEN=";
	public static final String FLAG_NAME = "FOLDER=";
	private int fileCounter = 0;
	private TCComponent projForm;

	public OpenPCBOperation3(AbstractAIFApplication app, String opName) {
		super(app, opName);
	}

	@Override
	public boolean init() throws Exception {
		InterfaceAIFComponent target = app.getTargetComponent();
		String type = target == null ? null : target.getType();
		System.out.println("Select: " + target + ", " + type);
		if (target == null || !(target instanceof TCComponentItemRevision)) {
			throw new TCException(ProcessLocale.getString("selectpcbrev.INFO"));
		}
		this.projRev = (TCComponentItemRevision) target;
		this.projVersion = this.projRev.getStringProperty("item_revision_id");
		this.projForm = projRev.getRelatedComponent("IMAN_master_form_rev");
		String[] prefVals = KUtil.getPrefVals(session, PREF_PCB);
		int len = KUtil.getLen(prefVals);
		if (len < 2) {
			throw new TCException(String.format(ProcessLocale.getString("preferror.ERROR1"), PREF_PCB));
		}
		String folderPath = prefVals[0];
//		String property = "java.io.tmpdir";

        // 获取临时目录并打印。
//        String folderPath = System.getProperty(property)+"\\temp";
		System.out.println("Download to folder: " + folderPath);
		this.parentFolder = new File(folderPath);
		 
//        if(parentFolder.exists()) {
//        	System.out.println("文件夹存在");
//        	deleteFile(parentFolder);
//        }
		if (!parentFolder.isDirectory()) {
			if (!parentFolder.mkdirs()) {
				throw new TCException(String.format(ProcessLocale.getString("mkdir.ERROR1"), folderPath));
			}
		}
		for (int i = 1; i < len; i++) {
			if (prefVals[i].startsWith(FLAG_OPEN)) {
				this.exts = prefVals[i].substring(FLAG_OPEN.length()).toLowerCase().split(":");
				System.out.println(">> Open ext: " + Arrays.toString(exts));
				continue;
			}
			if (prefVals[i].startsWith(FLAG_NAME)) {
				this.folderNameConfig = prefVals[i].substring(FLAG_NAME.length());
				System.out.println(">> Folder Name: " + folderNameConfig);
				continue;
			}
			String[] config = prefVals[i].split(":", -1);
			if (KUtil.getLen(config) == 5) {
				String rel_proj_item = config[0];
				String type_item = config[1];
				String rel_rev_dataset = config[2];
				String suffix_dataset = config[3];
				String prop_rev_id = config[4];
				System.out.println(">> Add Config: " + prefVals[i]);
				PCBConfig pcbConfig = new PCBConfig(rel_proj_item, type_item, rel_rev_dataset, suffix_dataset, prop_rev_id);
				pcbConfigs.add(pcbConfig);
			} else {
				System.out.println(">> Error Pref Value: " + prefVals[i]);
			}
		}
		if (this.pcbConfigs.size() == 0) {
			throw new TCException(String.format(ProcessLocale.getString("preferror.ERROR1"), PREF_PCB));
		}
		return true;
	}
	
	private TCComponentItemRevision getRev(TCComponent item, String revId) throws Exception{
		if(item==null||!(item instanceof TCComponentItem)) {
			return null;
		}
		TCComponent[] revs = item.getReferenceListProperty("revision_list");
		System.out.println("Revs: "+Arrays.toString(revs));
		int len = KUtil.getLen(revs);
		TCComponent res = null;
		if(KUtil.isEmpty(revId)) {
			for(int i=0;i<len;i++) {
				TCComponent rev = revs[i];
				if(projVersion.equals(rev.getStringProperty("item_revision_id"))) {
					if(res==null) {
						res = rev;
					}else {
						int resSequenceId = res.getIntProperty("sequence_id");
						int tempSequenceId = rev.getIntProperty("sequence_id");
						if (resSequenceId < tempSequenceId) {
							res = rev;
							System.out.println("Find Later Sequence" + rev + "|" + tempSequenceId);
						}
					}
				}
			}
		}else {
			for(int i=0;i<len;i++) {
				TCComponent rev = revs[i];
				if(revId.equals(rev.getStringProperty("item_revision_id"))) {
					if(res==null) {
						res = rev;
					}else {
						int resSequenceId = res.getIntProperty("sequence_id");
						int tempSequenceId = rev.getIntProperty("sequence_id");
						if (resSequenceId < tempSequenceId) {
							res = rev;
							System.out.println("Find Later Sequence" + rev + "|" + tempSequenceId);
						}
					}
				}
			}
		}
		return (TCComponentItemRevision) res;
	}

	private void downloadFiles(File dir, PCBConfig config, List<File> filesToOpen) throws Exception {
		System.out.println("Get File: "+config);
		List<TCComponent> items = KUtil.getChildComponents(projRev, config.type_item, config.rel_proj_item);
		System.out.println(">> Find Items: "+items);
		String revID = null;
		if(!KUtil.isEmpty(config.prop_revID)) {
			revID = this.projForm.getStringProperty(config.prop_revID);
			System.out.println(">> Assigned Rev ID: "+revID);
			if(KUtil.isEmpty(revID)) {
				throw new TCException(String.format(ProcessLocale.getString("revidempty.ERROR1"), config.prop_revID));
			}
		}
		int itemCnt = items==null?0:items.size();
		for (int c = 0;c<itemCnt;c++) {
			TCComponent item = items.get(c);
			System.out.println((c+1)+". Check Item: "+item);
			TCComponentItemRevision rev = getRev(item, revID);
			System.out.println("Find Revision: "+rev);
			if(rev==null) {
				continue;
			}
			List<TCComponentDataset> datasets = getDatasets(rev, config.rel_rev_dataset, config.suffix_dataset);
			System.out.println("Find Datasets: "+datasets);
			int size= datasets==null?0:datasets.size();
			for(int i=0;i<size;i++) {
				TCComponentDataset dataset = datasets.get(i);
				File[] files = dataset.getFiles("", dir.getAbsolutePath());
				System.out.println("Download Files: "+Arrays.toString(files));
				checkOpen(files, filesToOpen);
			}
		}
	}

	private void checkOpen(File[] files, List<File> filesToOpen) {
		int fileCnt  = KUtil.getLen(files);
		if(fileCnt==0) {
			return;
		}
		fileCounter+=fileCnt;
		int extCnt = KUtil.getLen(exts);
		if(extCnt==0) {
			return;
		}
		for(int j=0;j<fileCnt;j++) {
			String filePath = files[j].getAbsolutePath();
			int ind = filePath.lastIndexOf('.');
			String ext = filePath.substring(ind+1);
			if(!KUtil.isEmpty(ext)&&KUtil.inArray(ext.toLowerCase(), exts)>=0) {
				filesToOpen.add(files[j]);
			}
		}
	}

	private List<TCComponentDataset> getDatasets(TCComponent rev, String rel, String dsNamePrefix) throws Exception {
		if (rev == null) {
			return null;
		}
		rev.refresh();
		List<TCComponentDataset> res = new ArrayList<TCComponentDataset>();
		TCComponent[] kids = rev.getRelatedComponents(rel);
		for (TCComponent c : kids) {
			if ((c instanceof TCComponentDataset) && c.getProperty("object_name").toLowerCase().endsWith(dsNamePrefix.toLowerCase())) {
				res.add((TCComponentDataset) c);
			}
		}
		return res;
	}

	@Override
	public void execute() throws Exception {
		session.setStatus(ProcessLocale.getString("initfolder.STATUS"));
		// get folder
		List<TCComponentDataset> folderNameDatasets = null;
		File dir = this.parentFolder;
		if (!KUtil.isEmpty(folderNameConfig)) {
			int ind = folderNameConfig.indexOf(':');
			String folderName = null;
			if (ind > 0) {
				String rel_proj_dataset = folderNameConfig.substring(0, ind);
				String dsSuffix = folderNameConfig.substring(ind + 1);
				folderNameDatasets = getDatasets(projRev, rel_proj_dataset, dsSuffix);
				System.out.println("Find Proj Datasets: " + folderNameDatasets);
				if (folderNameDatasets.size() > 0) {
					folderName = folderNameDatasets.get(0).getProperty("object_name");
				}
			} else {
				folderName = folderNameConfig;
			}
			System.out.println("Target Folder: " + folderName);
			if (!KUtil.isEmpty(folderName)) {
				dir = new File(this.parentFolder.getAbsolutePath() + File.separator + folderName);
				if (!dir.isDirectory()) {
					if (!dir.mkdirs()) {
						throw new TCException(String.format(ProcessLocale.getString("mkdir.ERROR1"), dir.getAbsolutePath()));
					}
				}
			} else {
				throw new TCException(ProcessLocale.getString("getfoldername.ERROR"));
			}
		}
		FileUtils.deleteDirectory(dir);
		if (!dir.isDirectory()) {
			if (!dir.mkdirs()) {
				throw new TCException(String.format(ProcessLocale.getString("mkdir.ERROR1"), dir.getAbsolutePath()));
			}
		}
		// clear folder
//		String[] entries = dir.list();
//		for (String s : entries) {
//			File currentFile = new File(dir.getPath(), s);
//			if (!currentFile.delete()) {
//				throw new TCException(String.format(ProcessLocale.getString("deletefile.ERROR1"), currentFile.getAbsolutePath()));
//			}
//		}
		// download file
		ProcessLocale.getString("downloadfile.STATUS");
		List<File> filesToOpen = new ArrayList<File>();
		int configCnt = this.pcbConfigs.size();
		for (int i = 0; i < configCnt; i++) {
			PCBConfig pcbConfig = pcbConfigs.get(i);
			downloadFiles(dir, pcbConfig, filesToOpen);
		}
		if(folderNameDatasets!=null) {
			System.out.println("Download From Folder Name Config");
			for (TCComponentDataset ds : folderNameDatasets) {
				System.out.println("Dataset: "+ds);
				File[] files = ds.getFiles("", dir.getAbsolutePath());
				System.out.println("Download Files: "+Arrays.toString(files));
				checkOpen(files, filesToOpen);
			}
		}
		if(fileCounter==0) {
			throw new TCException(ProcessLocale.getString("nodataset.ERROR"));
		}
		// open files
		ProcessLocale.getString("openfile.status");
		System.out.println("Open Files: "+filesToOpen);
		for (File f : filesToOpen) {
			KUtil.openFile(f);
		}
	}
    public static Boolean deleteFile(File file) {
        //判断文件不为null或文件目录存在
        if (file == null || !file.exists()) {
            System.out.println("文件删除失败,请检查文件是否存在以及文件路径是否正确");
            return false;
        }
        //获取目录下子文件
        File[] files = file.listFiles();
        //遍历该目录下的文件对象
        for (File f : files) {
            //判断子目录是否存在子目录,如果是文件则删除
            if (f.isDirectory()) {
                //递归删除目录下的文件
                deleteFile(f);
            } else {
                //文件删除
                f.delete();
                //打印文件名
                System.out.println("文件名：" + f.getName());
            }
        }
        //文件夹删除
        file.delete();
        System.out.println("目录名：" + file.getName());
        return true;
    }

}
