package cn.net.connor.process.processcard;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.AIFComponentContext;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.kernel.TCComponentBOMLine;
import com.teamcenter.rac.kernel.TCComponentItemRevision;

import cn.net.connor.process.ProcessLocale;
import cn.net.connor.process.ProcessUtil;
import k.KOperation;
import k.util.KUtil;
import k.word.KWord;
import k.word.KWordUtil;

public class ProcessCardOperation_backup20200812 extends KOperation {

	private TCComponentBOMLine targetLine;
	protected String templateName_LBJLB;
	public static final String PREF_PREFIX_PARTLIST = "Connor_AVIC_GYKPSC_LBJLB_";
	public static final String PREF_PREFIX_PROCCARD = "Connor_AVIC_GYKPSC_GYKP_";
	public static final String PREF_PREFIX_KEYMEOP = "Connor_AVIC_GYKPSC_GJGX_";
	public static final String PREF_PREFIX_IMAGE = "Connor_AVIC_GYKPSC_GYFT_";
	public static final String PREF_PREFIX_IMAGE_GD = "Connor_AVIC_GYKPSC_GDFT_";
	public static final String PREF_GDTYPE = "Connor_AVIC_GDItem_Types";
	private boolean initFlag = false;
	private boolean exportLBJLB = false;
	private KWord lbjlb;
	private List<KWord> kWords = new ArrayList<KWord>();
	private List<KWord> gdWords = new ArrayList<KWord>();
	private TCComponentBOMLine processLine;
	private TCComponentItemRevision meTargetRev;
	private String[] gdTypes;

	public ProcessCardOperation_backup20200812(AbstractAIFApplication app, String opName) {
		super(app, opName);
	}

	public void setExportLBJLB(boolean flag) {
		this.exportLBJLB = flag;
	}

	@Override
	public boolean init() throws Exception {
		if (initFlag) {
			return true;
		}
		InterfaceAIFComponent target = app.getTargetComponent();
		if (target == null || !(target instanceof TCComponentBOMLine)) {
			KUtil.info(null, ProcessLocale.getString("selectmeop.INFO"));
			return false;
		}
		this.targetLine = (TCComponentBOMLine) target;
		System.out.println("Target line: " + this.targetLine);
		TCComponentItemRevision targetRev = this.targetLine.getItemRevision();
		if (!ProcessUtil.isMEOP(targetRev.getItem().getType())) {
			//if (!(targetRev instanceof TCComponentMEOPRevision)) {
			KUtil.info(null, ProcessLocale.getString("selectmeop.INFO"));
			return false;
		}
		// find process
		TCComponentBOMLine pLine = this.targetLine.parent();
		while (pLine != null) {
			if(ProcessUtil.isMEProcess(pLine.getItem().getType())) {
			//if (pLine.getItemRevision() instanceof TCComponentMEProcessRevision) {
				this.processLine = pLine;
				break;
			}
			pLine = pLine.parent();
		}
		System.out.println("Process line: " + this.processLine);
		if (this.processLine != null) {
			this.meTargetRev = (TCComponentItemRevision) this.processLine.getItemRevision().getRelatedComponent("IMAN_METarget");
			System.out.println("METarget: " + this.meTargetRev);
		}
		if(this.meTargetRev==null) {
			KUtil.info(null, ProcessLocale.getString("metargetnotfound.ERROR"));
			return false;
		}
		String prefSuffix = meTargetRev.getType();
		this.lbjlb = new KWord(session,  ProcessCardOperation_backup20200812.PREF_PREFIX_PARTLIST + prefSuffix);
		this.kWords.add(new KWord(session, ProcessCardOperation_backup20200812.PREF_PREFIX_PROCCARD + prefSuffix));
		String isKeyMeop = targetRev.getRelatedComponent("IMAN_master_form_rev").getStringProperty("ac5_KeyProcessOrNot");
		System.out.println("Is key meop: "+isKeyMeop);
		if("true".equals(isKeyMeop)) {
			this.kWords.add(new KWord(session, ProcessCardOperation_backup20200812.PREF_PREFIX_KEYMEOP+prefSuffix));
		}
		this.kWords.add(new KWord(session, ProcessCardOperation_backup20200812.PREF_PREFIX_IMAGE+prefSuffix));
		System.out.println(">> LBJLB Dataset: " + this.lbjlb.getTemplateDataset());
		this.templateName_LBJLB = this.lbjlb.getTemplateDataset().getProperty("object_name");
		// gd item types
		this.gdTypes = KUtil.getPrefVals(session, PREF_GDTYPE);
		System.out.println("GD Item Types: " + Arrays.toString(this.gdTypes));
		// gdft
		gdWords.add(new KWord(session, PREF_PREFIX_IMAGE_GD+prefSuffix));
		initFlag = true;
		return true;
	}
	
	@Override
	public void execute() throws Exception {
		System.out.println("Start Export Process Card...");
		System.out.println("Export LBJLB: " + this.exportLBJLB);
		if(this.exportLBJLB) {
			this.kWords.add(0, this.lbjlb);
		}
		String dsName =  ProcessLocale.getString("proccard.FILENAME") + "_" + KUtil.getDateStr(new Date(), "yyyyMMdd");
		String ftName =  ProcessLocale.getString("procimage.FILENAME") + "_" + KUtil.getDateStr(new Date(), "yyyyMMdd");
		KWordUtil.startWord();
		try {
			KWord.export(session, kWords, dsName, targetLine, meTargetRev, gdTypes);
			// export gd images
			AIFComponentContext[] cc = targetLine.getChildren();
			int cnt = KUtil.getLen(cc);
			for (int i = 0; i < cnt; i++) {
				TCComponentBOMLine cLine = (TCComponentBOMLine) cc[i].getComponent();
				String cItemType = cLine.getStringProperty("bl_item_object_type");
				if (KUtil.inArray(cItemType, gdTypes) >= 0) { // reorder child line by different types.
					System.out.println(String.format("Export GD Line: [%s](%s)", i + 1, cLine.toString(), cItemType));
					KWord.export(session, gdWords, ftName, cLine, meTargetRev, gdTypes);
				}
			}
		} finally {
			KWordUtil.closeWord();
		}
		System.out.println("Export Process Card Complete");
		KUtil.info(null, ProcessLocale.getString("proccardsuccess.INFO"));
	}

}
