package cn.net.connor.process.processcardmultiple;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.AIFComponentContext;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentBOMLine;
import com.teamcenter.rac.kernel.TCComponentDataset;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCException;
import com.teamcenter.rac.kernel.TCSession;

import cn.net.connor.process.ProcessLocale;
import cn.net.connor.process.ProcessUtil;
import k.KOperation;
import k.util.KUtil;
import k.word.KWord;
import k.word.KWordUtil;

public class MultiProcessCardOperation extends KOperation {

	public static final String PREF_PREFIX_SUMMARY = "Connor_AVIC_GYKPSC_CATE_";
	public static final String PREF_FLAG_GD = "GD#";
	public static final String PREF_FLAG_KEYMEOP = "KEY#";
	public static final String PREF_GDTYPE = "Connor_AVIC_GDItem_Types";
	private boolean initFlag = false;
	private String[] gdTypes;
	private String prefPrefix;
	private boolean isGeneralExport;
	protected List<MultiProcessCardBean> exportBeans = new ArrayList<MultiProcessCardBean>();

	public MultiProcessCardOperation(AbstractAIFApplication app, String opName, String prefName) {
		super(app, opName);
		this.isGeneralExport = !KUtil.isEmpty(prefName);
		this.prefPrefix = KUtil.isEmpty(prefName) ? PREF_PREFIX_SUMMARY : prefName;
	}

	@Override
	public boolean init() throws Exception {
		if (initFlag) {
			return true;
		}
		InterfaceAIFComponent[] targets = app.getTargetComponents();
		int len = KUtil.getLen(targets);
		for(int i=0; i<len; i++) {
			MultiProcessCardBean bean = checkTarget(session, targets[i], prefPrefix, isGeneralExport);
			if(bean!=null) {
				exportBeans.add(bean);
			}
		}
		if(exportBeans.size()==0) {
			KUtil.info(null, ProcessLocale.getString("processcardselect.ERROR"));
			return false;
		}
		// gd item types
		this.gdTypes = KUtil.getPrefVals(session, PREF_GDTYPE);
		System.out.println("GD Item Types: " + Arrays.toString(this.gdTypes));
		initFlag = true;
		return true;
	}

	protected static MultiProcessCardBean checkTarget(TCSession session, InterfaceAIFComponent target, String prefPrefix, boolean isGeneralExport) throws Exception {
		if (target == null || !(target instanceof TCComponentBOMLine)) {
			return null;
		}
		TCComponentBOMLine targetLine = (TCComponentBOMLine) target;
		TCComponentBOMLine processLine = null;
		TCComponentItemRevision meTargetRev = null;
		String dsNameFormat = null;
		KWord lbjlb = null;
		String templateName_LBJLB = null;
		List<String> gdWordsPref = new ArrayList<String>();
		List<KWord> kWords = new ArrayList<KWord>();
		TCComponentDataset oldExport = null;
		System.out.println("Check Target line: " + targetLine);
		TCComponentItemRevision targetRev = targetLine.getItemRevision();
		if (!ProcessUtil.isMEOP(targetRev.getItem().getType())) {
			System.out.println(">> Not Meop");
			return null;
		}
		// find process
		TCComponentBOMLine pLine = targetLine.parent();
		while (pLine != null) {
			if (ProcessUtil.isMEProcess(pLine.getItem().getType())) {
				processLine = pLine;
				break;
			}
			pLine = pLine.parent();
		}
		System.out.println("Process line: " + processLine);
		if (processLine != null) {
			meTargetRev = (TCComponentItemRevision) processLine.getItemRevision().getRelatedComponent("IMAN_METarget");
			System.out.println("METarget: " + meTargetRev);
		}
		if (meTargetRev == null) {
			System.out.println(">> Metarget Not Found");
			return null;
		}
		String prefSuffix = meTargetRev.getType();
		String isKeyMeop = targetRev.getRelatedComponent("IMAN_master_form_rev").getStringProperty("ac5_KeyProcessOrNot");
		System.out.println("Is key meop: " + isKeyMeop);
		// category
		{
			String prefName = prefPrefix + targetRev.getType();
			System.out.println("Read Category Pref: " + prefName);
			String[] cates = KUtil.getPrefVals(session, prefName);
			int len = KUtil.getLen(cates);
			if (len < 1) {
				KUtil.info(null, String.format(ProcessLocale.getString("preferror.ERROR1"), prefName));
				return null;
			}
			if (isGeneralExport) {
				dsNameFormat = cates[0];
			} else {
				try {
					lbjlb = new KWord(session, cates[0] + prefSuffix);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			for (int i = 1; i < len; i++) {
				String catePref = cates[i] + prefSuffix;
				if (catePref.startsWith(PREF_FLAG_GD)) {
					gdWordsPref.add(catePref.substring(PREF_FLAG_GD.length()));
				} else if (catePref.startsWith(PREF_FLAG_KEYMEOP)) {
					if ("true".equals(isKeyMeop)) {
						kWords.add(new KWord(session, catePref.substring(PREF_FLAG_KEYMEOP.length())));
					}
				} else {
					kWords.add(new KWord(session, catePref));
				}
			}
			if (lbjlb != null) {
				System.out.println(">> LBJLB Dataset: " + lbjlb.getTemplateDataset());
				templateName_LBJLB = lbjlb.getTemplateDataset().getProperty("object_name");
			}

		}
		TCComponentItemRevision gxRev = targetLine.getItemRevision();
		System.out.println("MEOP Rev: " + gxRev);
		TCComponent gxForm = gxRev.getRelatedComponent("IMAN_master_form_rev");
		String profession = gxForm.getStringProperty("ac5_Profession");
		String opNumber = gxForm.getStringProperty("ac5_OperationNumber");
		String name = gxRev.getStringProperty("object_name");
		System.out.println("profession = " + profession);
		System.out.println("operation number = " + opNumber);
		System.out.println("name = " + name);
		if (profession == null) {
			profession = "";
		}
		if (opNumber == null) {
			opNumber = "";
		}
		boolean gxRevReleased = KUtil.isReleased(gxRev);
		if(gxRevReleased) {
			System.out.println(">> Meop Released");
			return null;
		}
		boolean oldExportReleased = false;
		String checkExit = opNumber + name;
		if (!KUtil.isEmpty(checkExit)) {
			oldExport = findDataset(gxRev, "MSWordX", checkExit, KWord.REL_REV_DOC);
			if (oldExport != null) {
				oldExportReleased = KUtil.isReleased(oldExport);
			}
		}
		if (oldExport != null && gxRevReleased != oldExportReleased) {
			throw new TCException(ProcessLocale.getString("oldexportstatus.ERROR") + ": " + gxRev);
		}
		return new MultiProcessCardBean(targetLine, processLine, meTargetRev, dsNameFormat, lbjlb, templateName_LBJLB, gdWordsPref, kWords, oldExport, opNumber);
	}

	public static TCComponentDataset findDataset(TCComponent rev, String dsType, String dsName, String rel) throws Exception {
		if (rev == null || KUtil.isEmpty(dsName)) {
			return null;
		}
		System.out.println("Check Dataset: " + rev + " -> " + dsName + "|" + dsType + "|" + rel);
		rev.refresh();
		TCComponent[] kids = rev.getRelatedComponents(rel);
		System.out.println("Child: " + Arrays.toString(kids));
		for (TCComponent c : kids) {
			if (c.getType().equals(dsType) && c.toString().contains(dsName)) {
				System.out.println("Find Exist: " + c);
				return (TCComponentDataset) c;
			}
		}
		return null;
	}

	@Override
	public void execute() throws Exception {
		int exportCnt = this.exportBeans.size();
		System.out.println("Start Export Process Card... "+exportCnt);
		KWordUtil.startWord();
		try {
			for (int i = 0; i < exportCnt; i++) {
				MultiProcessCardBean bean = this.exportBeans.get(i);
				export(bean);
			} 
		} finally {
			KWordUtil.closeWord();
		}
		session.refreshComponentsByType("CfgBlAttachmentLine");
		session.refreshComponentsByType("MSWordX");
		System.out.println("Export Process Card Complete");
		KUtil.info(null, ProcessLocale.getString("proccardsuccess.INFO"));
	}

	protected void export(MultiProcessCardBean bean) throws Exception{
		System.out.println("Export "+bean.targetLine);
		System.out.println("ExportLBJLB -> " + bean.isExportLBJLB());
		if (bean.isExportLBJLB() && bean.lbjlb != null) {
			bean.kWords.add(0, bean.lbjlb);
		}
		String dsName = formatName(bean);
		List<KWord> combinelist = new ArrayList<KWord>();
		List<TCComponentBOMLine> targetLines = new ArrayList<TCComponentBOMLine>();
		List<TCComponentItemRevision> meTargetRevs = new ArrayList<TCComponentItemRevision>();
		List<String[]> item2TypesList = new ArrayList<String[]>();
		int size = bean.kWords.size();
		for (int i = 0; i < size; i++) {
			combinelist.add(bean.kWords.get(i));
			targetLines.add(bean.targetLine);
			meTargetRevs.add(bean.meTargetRev);
			item2TypesList.add(gdTypes);
		}
		size = bean.gdWordsPref.size();
		if (size > 0) {
			// export gd images
			AIFComponentContext[] cc = bean.targetLine.getChildren();
			int cnt = KUtil.getLen(cc);
			for (int i = 0; i < cnt; i++) {
				TCComponentBOMLine cLine = (TCComponentBOMLine) cc[i].getComponent();
				String cItemType = cLine.getStringProperty("bl_item_object_type");
				// System.out.println(cLine+" >> "+cItemType);
				if (KUtil.inArray(cItemType, gdTypes) >= 0) { // reorder child line by different types.
					System.out.println(String.format("Export GD Line: [%s](%s)", i + 1, cLine.toString(), cItemType));
					for (int j = 0; j < size; j++) {
						combinelist.add(new KWord(session, bean.gdWordsPref.get(j)));
						targetLines.add(cLine);
						meTargetRevs.add(bean.meTargetRev);
						item2TypesList.add(gdTypes);
					}
				}
			}
		}
		KWord.export2(session, combinelist, dsName, bean.targetLine, targetLines, meTargetRevs, !isGeneralExport);
	}

	private String formatName(MultiProcessCardBean bean) throws Exception {
		String dsName;
		if (KUtil.isEmpty(bean.dsNameFormat)) {
			dsName = ProcessLocale.getString("proccard.FILENAME");// + KUtil.getDateStr(new Date(), "yyyyMMdd");
			// String ftName = ProcessLocale.getString("procimage.FILENAME") + "_" +
			// KUtil.getDateStr(new Date(), "yyyyMMdd");
			{
				TCComponentItemRevision gxRev = bean.targetLine.getItemRevision();
				System.out.println("MEOP Rev: " + gxRev);
				TCComponent gxForm = gxRev.getRelatedComponent("IMAN_master_form_rev");
				String profession = gxForm.getStringProperty("ac5_Profession");
				String opNumber = gxForm.getStringProperty("ac5_OperationNumber");
				String name = gxRev.getStringProperty("object_name");
				System.out.println("profession = " + profession);
				System.out.println("operation number = " + opNumber);
				System.out.println("name = " + name);
				if (profession == null) {
					profession = "";
				}
				if (opNumber == null) {
					opNumber = "";
				}
				dsName = profession + opNumber + name + dsName;
				List<TCComponent> datasets = KUtil.getChildComponents(gxRev, "MSWordX", KWord.REL_REV_DOC, dsName);
				System.out.println(">> find datasets: " + datasets);
				TCComponent latestDataset = KUtil.getLatestComponent(datasets);
				System.out.println(">> latest: " + latestDataset);
				if (latestDataset != null) {
					dsName = latestDataset.getStringProperty("object_name");
				} else {
					dsName = dsName + KUtil.getDateStr(new Date(), "yyyyMMdd");
				}
			}
		} else {
			dsName = bean.dsNameFormat;
			int ind = dsName.lastIndexOf(':');
			String time = "";
			if (ind > 0) {
				time = new SimpleDateFormat(dsName.substring(ind + 1)).format(new Date());
				dsName = dsName.substring(0, ind);
			}
			TCComponentItemRevision rev = bean.targetLine.getItemRevision();
			String value = KUtil.getConfigPropValue(rev, dsName);
			dsName = value + time;
		}
		System.out.println("Dataset Name: " + dsName);
		return dsName;
	}

}
