package cn.net.connor.process.proctask;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumnModel;

import com.teamcenter.rac.aifrcp.AIFUtility;
import com.teamcenter.rac.util.MessageBox;

import cn.net.connor.process.ProcessLocale;
import k.KDialog;
import k.KDialogController;
import k.KOperation;
import k.KOperation2;
import k.util.KUtil;

@SuppressWarnings("serial")
public class ProcTaskDialog extends KDialog {

	public static final int COL_BOOLEAN = 1;
	protected DefaultTableModel tableModel;
	protected JTable table;
	private ProcTaskDialogController controller;
	protected List<JCheckBox> filterBoxList = new ArrayList<JCheckBox>();
	protected JButton b_selectPeople;
	protected JButton b_assignTask;
	protected JButton b_cancel;

	public ProcTaskDialog(KDialogController controller) {
		super(controller);
		this.controller = (ProcTaskDialogController) controller;
	}

	@Override
	protected void addListeners() {
		b_cancel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				disposeDialog();
			}
		});
		b_selectPeople.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					controller.selectPeople();
				} catch (Exception e1) {
					e1.printStackTrace();
					MessageBox.post(ProcTaskDialog.this, e1);
				}
			}
		});
		b_assignTask.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				new KOperation2(AIFUtility.getCurrentApplication(), ProcessLocale.getString("assigntask.STATUS")) {
					
					@Override
					public boolean init() throws Exception {
						b_assignTask.setEnabled(false);
						b_selectPeople.setEnabled(false);
						table.setEnabled(false);
						return true;
					}
					
					@Override
					public void execute() throws Exception {
						try {
							controller.assignTask();
						} catch (Exception e1) {
							e1.printStackTrace();
							MessageBox.post(ProcTaskDialog.this, e1);
						}
					}

					@Override
					public void clearCache() throws Exception {
						b_assignTask.setEnabled(true);
						b_selectPeople.setEnabled(true);
						table.setEnabled(true);
					}
				}.executeModeless();
			}
		});
		this.tableModel.addTableModelListener(new TableModelListener() { // table选择监听
			@Override
			public void tableChanged(TableModelEvent e) {
				if (e.getColumn() == COL_BOOLEAN) {
					int row = e.getFirstRow();
					ProcTaskRow bean = (ProcTaskRow) tableModel.getValueAt(row, 0);
					String selected = tableModel.getValueAt(row, COL_BOOLEAN).toString();
					if ("true".equals(selected)) { // 勾选当前行
						bean.setSelected(true);
					} else {
						bean.setSelected(false);
					}
				}
			}
		});
	}

	@Override
	protected void initUI() {
		this.setTitle(ProcessLocale.getString("proctask.TITLE"));
		this.setPreferredSize(new Dimension(900, 600));
		this.setLayout(new BorderLayout());
		this.add(BorderLayout.NORTH, initFilterPanel());
		this.add(BorderLayout.CENTER, initTablePanel());
		this.add(BorderLayout.SOUTH, initBtnPanel());
		b_assignTask.setEnabled(false);
		b_selectPeople.setEnabled(false);
		for (JCheckBox jCheckBox : filterBoxList) {
			jCheckBox.setEnabled(false);
		}
		new KOperation(AIFUtility.getCurrentApplication(), ProcessLocale.getString("loaddata.STATUS")) {

			@Override
			public boolean init() throws Exception {
				return true;
			}

			@Override
			public void execute() throws Exception {
				try {
					controller.readData();
					if (controller.receiveProcess()) {
						b_assignTask.setEnabled(true);
						b_selectPeople.setEnabled(true);
					}
					for (JCheckBox jCheckBox : filterBoxList) {
						jCheckBox.setEnabled(true);
					}
				} catch (Exception e) {
					e.printStackTrace();
					MessageBox.post(ProcTaskDialog.this, e);
				}
			}
		}.executeModeless();
	}

	private Component initBtnPanel() {
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER, 15, 5));
		panel.add(this.b_selectPeople = new JButton(ProcessLocale.getString("selecttaskassignee")));
		panel.add(this.b_assignTask = new JButton(ProcessLocale.getString("assigntask")));
		panel.add(this.b_cancel = new JButton(ProcessLocale.getString("cancel")));
		return panel;
	}

	private Component initFilterPanel() {
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 20, 5));
		panel.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				Component lc = panel.getComponent(panel.getComponentCount() - 1);
				int h = lc.getY() + lc.getSize().height + 5;
				int w = panel.getSize().width;
				panel.setPreferredSize(new Dimension(w, h));
				panel.revalidate();
			}
		});
		for (String filterType : controller.filterTypeMap.keySet()) {
			JCheckBox check = new JCheckBox(filterType);
			panel.add(check);
			filterBoxList.add(check);
			check.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					try {
						controller.filterType();
					} catch (Exception e) {
						e.printStackTrace();
						MessageBox.post(ProcTaskDialog.this, e);
					}
				}
			});
		}
		return panel;
	}

	private Component initTablePanel() {
		this.tableModel = new DefaultTableModel();
		this.table = new JTable(tableModel) {
			@Override
			public boolean isCellEditable(int row, int column) {
				if (column == COL_BOOLEAN)
					return true;
				return false;
			}

			@Override
			public Class<?> getColumnClass(int column) {
				if (column == COL_BOOLEAN) {
					return Boolean.class;
				}
				return super.getColumnClass(column);
			}
		};
		table.setRowHeight(23);
		tableModel.setDataVector(null, controller.tableHeaders);
		table.getTableHeader().setReorderingAllowed(false);
		table.getTableHeader().setDefaultRenderer(new CheckBoxTableHeaderRenderer(table, COL_BOOLEAN, true));
		table.setDefaultRenderer(Object.class, new ColorTableRenderer());
		int colCnt = KUtil.getLen(controller.tableHeaders);
		TableColumnModel colModel = this.table.getColumnModel();
		for (int i = 0; i < colCnt; i++) {
			colModel.getColumn(i).setPreferredWidth(controller.tableHeaderWidth[i]);
		}
		colModel.getColumn(0).setMinWidth(0);
		colModel.getColumn(0).setMaxWidth(0);
		colModel.getColumn(1).setMinWidth(50);
		colModel.getColumn(1).setMaxWidth(50);
		if (colCnt > 6) {
			table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		}
		JScrollPane scroll = new JScrollPane(table);
		scroll.setBorder(null);
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(BorderLayout.CENTER, scroll);
		panel.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));
		return panel;
	}

	public static class ColorTableRenderer extends DefaultTableCellRenderer {

		public static final Color DEFBLUE = new Color(51, 153, 255);
		public static final Color RED = new Color(255, 59, 48);
		public static final Color GREEN = new Color(158, 234, 106);
		public static final Color YELLOW = new Color(247, 239, 20);

		@Override
		public Component getTableCellRendererComponent(JTable t, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			if (hasFocus) {
				setBackground(DEFBLUE);
			} else {
				Object obj = t.getValueAt(row, 0);
				if (obj instanceof ProcTaskRow) {
					if (((ProcTaskRow) obj).hasWorkflow()&&((ProcTaskRow) obj).hasAssignee()) {
						setBackground(GREEN);
					} else if (((ProcTaskRow) obj).hasAssignee()) {
						setBackground(YELLOW);
					} else {
						setBackground(RED);
					}
				} else {
					setBackground(Color.WHITE);
				}
			}
			return super.getTableCellRendererComponent(t, value, isSelected, hasFocus, row, column);
		}

	}
}
