package cn.net.connor.process.proctask;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JCheckBox;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.AIFComponentContext;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.kernel.SoaUtil;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentBOMLine;
import com.teamcenter.rac.kernel.TCComponentItem;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCComponentTask;
import com.teamcenter.rac.kernel.TCComponentTaskTemplate;
import com.teamcenter.rac.kernel.TCComponentUser;
import com.teamcenter.rac.kernel.TCException;
import com.teamcenter.rac.kernel.TCHandlerType;
import com.teamcenter.services.rac.workflow.WorkflowService;
import com.teamcenter.services.rac.workflow._2008_06.Workflow.ContextData;
import com.teamcenter.services.rac.workflow._2008_06.Workflow.InstanceInfo;

import cn.net.connor.process.ProcessLocale;
import k.KDialogController;
import k.util.KOrgDialog;
import k.util.KUtil;

public class ProcTaskDialogController extends KDialogController {

	public static final String PREF_TARGET_TYPE = "Connor_AVIC_GYFGPZ_LXSX";
	public static final String PREF_PROPS = "Connor_AVIC_GYFGPZ_SXPZ";
	public static final String PREF_HANDLER = "Connor_AVIC_GYFGPZ_HANDLE"; // CONNOR-check-form-attribute
	public static final String PREF_WORKFLOW = "Connor_AVIC_GYFGPZ_WORKFLOW"; // CONNOR-check-form-attribute
	public static final String COLNAME_TASKASSIGNEE = ProcessLocale.getString("colname_taskassignee");
	private String[] targetTypes;
	protected String[] tableHeaders;
	protected int[] tableHeaderWidth;
	private String[] tableProps;
	private ProcTaskDialog dialog;
	protected Map<String, List<String>> filterTypeMap = new LinkedHashMap<String, List<String>>();
	private TCComponentBOMLine bomline;
	private int colIndex_TaskAssignee = -1;
	private List<ProcTaskRow> rowDatas = new ArrayList<ProcTaskRow>();
	private String workflowName;

	public ProcTaskDialogController(AbstractAIFApplication app) {
		super(app);
	}

	@Override
	public boolean init() throws Exception {
		this.dialog = (ProcTaskDialog) aifDialog;
		this.workflowName = KUtil.getPrefVal(session, PREF_WORKFLOW);
		System.out.println("Workflow Name: " + workflowName);
		if (KUtil.isEmpty(workflowName)) {
			throw new TCException(String.format(ProcessLocale.getString("prefnotconfiged.ERROR1"), PREF_WORKFLOW));
		}
		this.targetTypes = KUtil.getPrefVals(session, PREF_TARGET_TYPE);
		System.out.println("Target Type: " + Arrays.toString(targetTypes));
		if (KUtil.getLen(targetTypes) == 0) {
			KUtil.warn(dialog, String.format(ProcessLocale.getString("prefnotconfiged.ERROR1"), PREF_TARGET_TYPE));
			return false;
		}
		InterfaceAIFComponent comp = app.getTargetComponent();
		System.out.println("Select: " + comp);
		if (comp != null) {
			if (comp instanceof TCComponentBOMLine) {
				this.bomline = (TCComponentBOMLine) comp;
			}
		}
		if (bomline == null) {
			return false;
		}
		int typeCnt = KUtil.getLen(targetTypes);
		for (int i = 0; i < typeCnt; i++) {
			String type = targetTypes[i];
			int ind = type.indexOf(':');
			if (ind > 0) {
				String cate = type.substring(0, ind);
				type = type.substring(ind + 1);
				targetTypes[i] = type;
				List<String> ll;
				if (filterTypeMap.containsKey(cate)) {
					ll = filterTypeMap.get(cate);
				} else {
					ll = new ArrayList<String>();
					filterTypeMap.put(cate, ll);
				}
				ll.add(type);
			}
		}
		String[] tableConfigs = KUtil.getPrefVals(session, PREF_PROPS);
		if (KUtil.getLen(tableConfigs) == 0) {
			KUtil.warn(dialog, String.format(ProcessLocale.getString("prefnotconfiged.ERROR1"), PREF_PROPS));
			return false;
		}
		int colCnt = KUtil.getLen(tableConfigs);
		tableHeaders = new String[colCnt + 2];
		tableHeaderWidth = new int[colCnt + 2];
		tableHeaders[0] = "Hidden";
		tableHeaderWidth[0] = 0;
		tableHeaders[1] = "";
		tableHeaderWidth[1] = 50;
		tableProps = new String[colCnt];
		for (int i = 0; i < colCnt; i++) {
			int colIndex = i + 2;
			String config = tableConfigs[i];
			int ind = config.lastIndexOf('=');
			if (ind > 0) {
				tableHeaders[colIndex] = config.substring(0, ind);
				tableProps[i] = config.substring(ind + 1);
			} else {
				tableHeaders[colIndex] = config;
				tableProps[i] = config;
			}
			int width = 0;
			while (tableHeaders[colIndex].length() > 0 && Character.isDigit(tableHeaders[colIndex].charAt(0))) {
				width = width * 10 + (tableHeaders[colIndex].charAt(0) - '0');
				tableHeaders[colIndex] = tableHeaders[colIndex].substring(1);
			}
			if (width < 1) {
				width = 120;
			}
			tableHeaderWidth[colIndex] = width;
			if (COLNAME_TASKASSIGNEE.equals(tableHeaders[colIndex])) {
				this.colIndex_TaskAssignee = colIndex;
			}
		}
		System.out.println("Task Assignee Col Index: " + this.colIndex_TaskAssignee);
		if (this.colIndex_TaskAssignee < 0) {
			throw new TCException(ProcessLocale.getString("taskassigneecolnotfound"));
		}
		return true;
	}

	private void addRow(TCComponentBOMLine line) throws Exception {
		ProcTaskRow bean = new ProcTaskRow(line, colIndex_TaskAssignee);
		this.rowDatas.add(bean);
		dialog.tableModel.addRow(bean.getRowData(tableProps, true, workflowName));
	}

	protected void readData() throws Exception {
		System.out.println("Load BOM: " + this.bomline + " | " + Arrays.toString(targetTypes));
		if (this.bomline == null) {
			return;
		}
		this.bomline.window().refresh();
		KUtil.setByPass(true);
		try {
			this.bomline.refresh();
			cycleBom("", bomline);
			// if (KUtil.inArray(getBomLineType(bomline), targetTypes) >= 0) {
			// addRow(bomline);
			// }
			// AIFComponentContext[] cc = this.bomline.getChildren();
			// int len = KUtil.getLen(cc);
			// for (int i = 0; i < len; i++) {
			// TCComponentBOMLine cLine = (TCComponentBOMLine) cc[i].getComponent();
			// String cType = getBomLineType(cLine); // cLine.getItem().getType();
			// System.out.println((i + 1) + ". " + cLine + " -> " + cType);
			// if (KUtil.inArray(cType, targetTypes) >= 0) {
			// addRow(cLine);
			// }
			// AIFComponentContext[] ccc = cLine.getChildren();
			// int len2 = KUtil.getLen(ccc);
			// for (int j = 0; j < len2; j++) {
			// TCComponentBOMLine ccLine = (TCComponentBOMLine) ccc[j].getComponent();
			// String ccType = getBomLineType(ccLine); // ccLine.getItem().getType();
			// System.out.println((i + 1) + "." + (j + 1) + " " + ccLine + " -> " + ccType);
			// if (KUtil.inArray(ccType, targetTypes) >= 0) {
			// addRow(ccLine);
			// }
			// }
			// }
		} finally {
			KUtil.setByPass(false);
		}
	}

	protected void cycleBom(String status, TCComponentBOMLine pline) throws Exception {
		if (pline == null) {
			return;
		}
		String lineType = getBomLineType(pline);
		String msg = status + " " + pline + " -> " + lineType;
		System.out.println(msg);
		if (KUtil.inArray(lineType, targetTypes) >= 0) {
			addRow(pline);
		}
		AIFComponentContext[] cc = pline.getChildren();
		int len = KUtil.getLen(cc);
		for (int i = 0; i < len; i++) {
			TCComponentBOMLine cLine = (TCComponentBOMLine) cc[i].getComponent();
			cycleBom(status + " > " + (i + 1), cLine);
			if (status.isEmpty()) {
				session.setStatus("( " + (i + 1) + " / " + len + " ) " + cLine);
			}
		}

	}

	private String getBomLineType(TCComponentBOMLine line) throws Exception {
		if (line == null) {
			return null;
		}
		TCComponentItem item = line.getItem();
		if (item == null) {
			throw new TCException(String.format(ProcessLocale.getString("itemnorfoundfrombomline.ERROR1"), line.toString()));
		}
		return item.getType();
	}

	protected boolean receiveProcess() throws Exception {
		String handlerName = KUtil.getPrefVal(session, PREF_HANDLER);
		System.out.println("Handler Name: " + handlerName);
		if (KUtil.isEmpty(handlerName)) {
			return true;
		}
		System.out.println(">> Check Process Info...");
		TCComponentItemRevision rev = bomline.getItemRevision();
		rev.refresh();
		TCComponent[] procList = rev.getReferenceListProperty("fnd0StartedWorkflowTasks");
		System.out.println(Arrays.toString(procList));
		if (procList != null) {
			int len = procList.length;
			for (int i = 0; i < len; i++) {
				TCComponentTask task = (TCComponentTask) procList[i];
				String taskType = task.getTaskType();
				System.out.println("Find Task: " + task + " |" + taskType);
				TCComponent handlerComp = task.getHandler(TCHandlerType.ACTION, TCComponentTaskTemplate.START_ACTION, handlerName);
				if (handlerComp != null) {
					System.out.println("Find Handler");
					return true;
				}
			}
		} else {
			System.out.println("process_stage_list is empty: " + rev);
		}
		return false;
	}

	public void selectPeople() throws Exception {
		// int[] rows = dialog.table.getSelectedRows();
		// System.out.println("Select Row: " + Arrays.toString(rows));
		// int len = rows == null ? 0 : rows.length;
		// if (len <= 0) {
		// return;
		// }
		Map<Integer, ProcTaskRow> selected = new HashMap<Integer, ProcTaskRow>();
		int rowCnt = dialog.table.getRowCount();
		for (int i = 0; i < rowCnt; i++) {
			if ((boolean) dialog.table.getValueAt(i, ProcTaskDialog.COL_BOOLEAN)) {
				ProcTaskRow bean = (ProcTaskRow) dialog.table.getValueAt(i, 0);
				if (bean.hasWorkflow()) {
					System.out.println("Workflow Existed: " + bean);
					throw new TCException(ProcessLocale.getString("taskstarted.ERROR"));
				}
				selected.put(i, bean);
			}
		}
		System.out.println("Select Row: " + selected);
		int len = selected.size();
		if (len <= 0) {
			return;
		}
		KUtil.setByPass(true);
		try {
			TCComponentUser user = new KOrgDialog("").getUser();
			System.out.println("Select User: " + user);
			String userName = user == null ? "" : user.getUserName();
			for (Integer row : selected.keySet()) {
				ProcTaskRow bean = selected.get(row);
				bean.setAssignee(userName, tableProps[colIndex_TaskAssignee - 2]);
				bean.refreshData(tableProps, true, workflowName);
				dialog.table.setValueAt(userName, row, colIndex_TaskAssignee);
			}
			// for (int i = 0; i < len; i++) {
			// int row = rows[i];
			// ProcTaskRow bean = (ProcTaskRow) dialog.table.getValueAt(row, 0);
			// bean.setAssignee(userName, tableProps[colIndex_TaskAssignee - 2]);
			// dialog.table.setValueAt(user, row, colIndex_TaskAssignee);
			// }
		} finally {
			KUtil.setByPass(false);
		}
		dialog.table.revalidate();
		dialog.table.repaint();
	}

	public void assignTask_old() throws Exception {
		List<ProcTaskRow> selected = new ArrayList<ProcTaskRow>();
		int rowCnt = dialog.table.getRowCount();
		for (int i = 0; i < rowCnt; i++) {
			if ((boolean) dialog.table.getValueAt(i, ProcTaskDialog.COL_BOOLEAN)) {
				ProcTaskRow bean = (ProcTaskRow) dialog.table.getValueAt(i, 0);
				if (bean.hasWorkflow()) {
					throw new TCException(ProcessLocale.getString("taskstarted.ERROR"));
				}
				selected.add(bean);
			}
		}
		System.out.println("Select Row: " + selected);
		int len = selected.size();
		if (len <= 0) {
			return;
		}
		KUtil.setByPass(true);
		try {
			for (int i = 0; i < len; i++) {
				ProcTaskRow bean = selected.get(i);
				bean.launchWorkflow(workflowName);
			}
		} finally {
			KUtil.setByPass(false);
		}
		dialog.table.revalidate();
		dialog.table.repaint();
		KUtil.info(dialog, ProcessLocale.getString("assigntask.SUCCESS"));
	}

	public void assignTask() throws Exception {
		Map<String, List<ProcTaskRow>> selected = new HashMap<String, List<ProcTaskRow>>();
		int rowCnt = dialog.table.getRowCount();
		for (int i = 0; i < rowCnt; i++) {
			if ((boolean) dialog.table.getValueAt(i, ProcTaskDialog.COL_BOOLEAN)) {
				ProcTaskRow bean = (ProcTaskRow) dialog.table.getValueAt(i, 0);
				if (bean.hasWorkflow()) {
					throw new TCException(ProcessLocale.getString("taskstarted.ERROR"));
				}
				// TCComponentUser userValue = (TCComponentUser) dialog.table.getValueAt(i,
				// colIndex_TaskAssignee);
				String user = (String) dialog.table.getValueAt(i, colIndex_TaskAssignee);
				if (KUtil.isEmpty(user)) {
					throw new TCException(ProcessLocale.getString("selectworkflowuser.ERROR"));
				}
				if (selected.containsKey(user)) {
					selected.get(user).add(bean);
				} else {
					List<ProcTaskRow> list = new ArrayList<ProcTaskRow>();
					list.add(bean);
					selected.put(user, list);
				}
			}
		}
		System.out.println("Select Row: " + selected);
		int len = selected.size();
		if (len <= 0) {
			return;
		}
		KUtil.setByPass(true);
		try {
			int cnt = selected.size();
			int ind = 1;
			for (String user : selected.keySet()) {
				session.setStatus("( " + (ind++) + " / " + cnt + " ) " + user);
				List<ProcTaskRow> beans = selected.get(user);
				launchWorkflow(workflowName, beans);
			}
		} finally {
			KUtil.setByPass(false);
		}
		dialog.table.revalidate();
		dialog.table.repaint();
		KUtil.info(dialog, ProcessLocale.getString("assigntask.SUCCESS"));
	}

	public void launchWorkflow(String workflowName, List<ProcTaskRow> beans) throws Exception {
		if (beans == null || beans.size() == 0) {
			return;
		}
		System.out.println("Start workflow: " + beans);
		List<String> uids = new ArrayList<String>();
		for (ProcTaskRow bean : beans) {
			String uid = bean.getBomline().getItemRevision().getUid();
			if (!uids.contains(uid)) {
				uids.add(uid);
			}
		}
		System.out.println(">> Uid: " + uids);
		TCComponentItemRevision rev = beans.get(0).getBomline().getItemRevision();
		String name = rev.toString();
		String subject = rev.toString();
		String observerKey = "";
		String desc = "";
		WorkflowService wfService = WorkflowService.getService(KUtil.session);
		ContextData contextData = new ContextData();
		int attCnt = uids.size();
		contextData.attachmentCount = attCnt;
		contextData.attachments = uids.toArray(new String[] {});
		int[] types = new int[attCnt];
		for (int i = 0; i < attCnt; i++) {
			types[i] = 1;
		}
		contextData.attachmentTypes = types;
		contextData.subscribeToEvents = false;
		contextData.subscriptionEventCount = 0;
		contextData.processTemplate = workflowName;
		InstanceInfo resp = wfService.createInstance(false, observerKey, name, subject, desc, contextData);
		SoaUtil.checkPartialErrors(resp.serviceData);
		for (ProcTaskRow bean : beans) {
			bean.setHasWorkflow(true);
		}
	}

	public void filterType() throws Exception {
		List<String> types = new ArrayList<String>();
		for (JCheckBox check : dialog.filterBoxList) {
			if (check.isSelected()) {
				types.addAll(this.filterTypeMap.get(check.getText()));
			}
		}
		System.out.println("Filter Type: " + types);
		dialog.tableModel.getDataVector().clear();
		if (types.size() == 0) {
			for (ProcTaskRow row : this.rowDatas) {
				dialog.tableModel.addRow(row.getRowData(tableProps, true, workflowName));
			}
		} else {
			for (ProcTaskRow row : this.rowDatas) {
				if (types.contains(row.getItemType())) {
					dialog.tableModel.addRow(row.getRowData(tableProps, true, workflowName));
				}
			}
		}
	}

}
