package cn.net.connor.process.proctask;

import java.util.Arrays;
import java.util.HashMap;

import com.teamcenter.rac.aif.kernel.AIFComponentContext;
import com.teamcenter.rac.kernel.SoaUtil;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentBOMLine;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCException;
import com.teamcenter.services.rac.workflow.WorkflowService;
import com.teamcenter.services.rac.workflow._2008_06.Workflow.ContextData;
import com.teamcenter.services.rac.workflow._2008_06.Workflow.InstanceInfo;

import cn.net.connor.process.ProcessLocale;
import k.util.KUtil;
import k.util.PropLocation;

public class ProcTaskRow {

	private TCComponentBOMLine bomline;
	private boolean hasWorkflow;
	private HashMap<String, TCComponent> markedComponents;
	private int colIndex_TaskAssignee;
	private boolean hasAssignee;
	private String itemType;
	Object[] data;

	public ProcTaskRow(TCComponentBOMLine cLine, int colIndex_TaskAssignee) throws Exception {
		this.bomline = cLine;
		this.itemType = bomline.getItem().getType();
		this.colIndex_TaskAssignee = colIndex_TaskAssignee;
		markedComponents = new HashMap<String, TCComponent>();
		PropLocation.initLocationMap(markedComponents, getBomline());
		TCComponentItemRevision rev = cLine.getItemRevision();
		if(rev!=null) {
			rev.refresh();
			AIFComponentContext[] proc = rev.whereReferencedByTypeRelation(null, new String[] {"IMAN_METarget"});
			System.out.println("Proc: "+rev+" -> "+Arrays.toString(proc));
			if(KUtil.getLen(proc)>0) {
				PropLocation.initMLocationMap(markedComponents, (TCComponentItemRevision)proc[0].getComponent());
			}
		}else{
			throw new TCException(String.format(ProcessLocale.getString("bomrevnotfound.ERROR1"), bomline.toString()));
		}
	}

	public boolean hasWorkflow() {
		return this.hasWorkflow;
	}
	
	public void setHasWorkflow(boolean b) {
		this.hasWorkflow = b;
	}

	public boolean hasAssignee() {
		return hasAssignee;
	}

	public String getItemType() {
		return this.itemType;
	}

	public void refreshData(String[] tableProps, boolean select, String workflowName) throws Exception {
		int len = KUtil.getLen(tableProps);
		data = new Object[len + 2];
		data[0] = this;
		data[1] = select;
		for (int i = 0; i < len; i++) {
			String[] config = tableProps[i].split("\\.");
			if (KUtil.getLen(config) != 2) {
				data[i + 2] = tableProps[i];
				continue;
			}
			String propLoc = config[0].toUpperCase();
			if (markedComponents.containsKey(propLoc)) {
				data[i + 2] = markedComponents.get(propLoc).getProperty(config[1]);
			//} else {
			//	data[i + 2] = tableProps[i];
			}
		}
		this.hasAssignee = !KUtil.isEmpty((String) data[colIndex_TaskAssignee]);
		TCComponent[] workflows = markedComponents.get(PropLocation.REV).getReferenceListProperty("fnd0AllWorkflows");
		int procCnt = KUtil.getLen(workflows);
		this.hasWorkflow = false;
		for(int i=0 ; i<procCnt;i++) {
			TCComponent template = workflows[i].getReferenceProperty("task_template");
			if(template!=null&&workflowName.equals(template.getStringProperty("object_name"))) {
				this.hasWorkflow = true;
				break;
			}
		}
//		this.hasWorkflow = KUtil.getLen(procs) > 0;
	}

	public Object[] getRowData(String[] tableProps, boolean select, String workflowName) throws Exception {
		if (data == null) {
			refreshData(tableProps, select, workflowName);
		}
		return data;
	}

	@Override
	public String toString() {
		return getBomline().toString();
	}

	public TCComponentBOMLine getBomline() {
		return bomline;
	}

	public void setSelected(boolean b) {
		System.out.println(b ? "Select" : "Unselect: " + bomline);
		data[1] = b;
	}

	public void setAssignee(String user, String propConfig) throws Exception {
		this.hasAssignee = !KUtil.isEmpty(user);
		String[] config = propConfig.split("\\.");
		if (KUtil.getLen(config) != 2) {
			return;
		}
		String propLoc = config[0].toUpperCase();
		if (markedComponents.containsKey(propLoc)) {
			TCComponent comp = markedComponents.get(propLoc);
			System.out.println("Set User: " + comp + "  " + config[1] + " -> " + user);
			comp.setStringProperty(config[1], user);
			comp.refresh();
		}
	}

	public void launchWorkflow(String workflowName) throws Exception{
		TCComponentItemRevision rev = bomline.getItemRevision();
		String name = rev.toString();
		String subject = rev.toString();
		String observerKey = "";
		String desc = "";
		System.out.println("Start workflow: "+rev);
		WorkflowService wfService = WorkflowService.getService(KUtil.session);
		ContextData contextData = new ContextData();
		contextData.attachmentCount = 1;
		contextData.attachments = new String[] { rev.getUid() };
		contextData.attachmentTypes = new int[] { 1 };
		contextData.subscribeToEvents = false;
		contextData.subscriptionEventCount = 0;
		contextData.processTemplate = workflowName;
		InstanceInfo resp = wfService.createInstance(false, observerKey, name, subject, desc, contextData);
		SoaUtil.checkPartialErrors(resp.serviceData);
		this.hasWorkflow = true;
	}
}
