package cn.net.connor.process.tsgcexport;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.teamcenter.rac.aif.kernel.AIFComponentContext;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentBOMLine;
import com.teamcenter.rac.kernel.TCComponentBOPWindow;
import com.teamcenter.rac.kernel.TCComponentBOPWindowType;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCSession;

import cn.net.connor.process.ProcessUtil;
import k.util.KUtil;
import k.util.PropLocation;

public class TSGCBean {

	private TCComponentItemRevision meTargetRev;
	private TCComponentItemRevision meopRev;
	private TCComponentItemRevision tsgcRev;
	protected Map<String, TCComponent> markedComponents = new HashMap<String, TCComponent>();

	public TSGCBean(TCComponentItemRevision meTargetRev, TCComponentItemRevision procRev, TCComponentItemRevision meopRev, TCComponentItemRevision tsgcRev) throws Exception{
		this.meTargetRev = meTargetRev;
		this.meopRev = meopRev;
		this.tsgcRev = tsgcRev;
		initLocationMap(markedComponents, tsgcRev, "");
		initLocationMap(markedComponents, procRev, "G");
		initLocationMap(markedComponents, meopRev, "P");
		initLocationMap(markedComponents, meTargetRev, "M");
	}
	
	public static void initLocationMap(Map<String, TCComponent> markedComponents, TCComponentItemRevision rev, String prefix) throws Exception {
		if (markedComponents == null || rev == null) {
			return;
		}
		markedComponents.put(prefix+PropLocation.ITEM, rev.getItem());
		markedComponents.put(prefix+PropLocation.REV, rev);
		markedComponents.put(prefix+PropLocation.ITEMMASTER, rev.getItem().getRelatedComponent("IMAN_master_form"));
		markedComponents.put(prefix+PropLocation.REVMASTER, rev.getRelatedComponent("IMAN_master_form_rev"));
	}
	
	public static void parse(TCComponentItemRevision tsgcRev, List<TSGCBean> tsgcBeans) throws Exception{
		if(tsgcRev==null) {
			return;
		}
		System.out.println("search reference: "+tsgcRev);
		List<TCComponentItemRevision> meopRevs = new ArrayList<TCComponentItemRevision>();
		TCComponent[] meops = tsgcRev.whereUsed(TCComponentItemRevision.WHERE_USED_ALL);
		int len = KUtil.getLen(meops);
		for(int i=0; i<len; i++) {
			TCComponent meop = meops[i];
			if(!(meop instanceof TCComponentItemRevision)) {
				continue;
			}
			TCComponentItemRevision meopRev = (TCComponentItemRevision) meop;
			if(ProcessUtil.isMEOP(meopRev.getItem().getType())) {
				meopRevs.add(meopRev);
			}
		}
		int meopCnt = meopRevs.size();
		System.out.println("find meop count: "+meopCnt);
		for(int i=0; i<meopCnt; i++) {
			TCComponentItemRevision meopRev = meopRevs.get(i);
			System.out.println("check meprocess: "+meopRev);
			TCComponent[] procs = meopRev.whereUsed(TCComponentItemRevision.WHERE_USED_ALL);
			int procCnt = KUtil.getLen(procs);
			for(int j=0;j<procCnt;j++) {
				TCComponent proc = procs[j];
				if(!(proc instanceof TCComponentItemRevision)) {
					continue;
				}
				TCComponentItemRevision procRev = (TCComponentItemRevision) proc;
				if(!ProcessUtil.isMEProcess(procRev.getItem().getType())) {
					continue;
				}
				TCComponent[] targets = procRev.getRelatedComponents("IMAN_METarget");
				System.out.println(">> "+procRev+" -> "+Arrays.toString(targets));
				int targetCnt = KUtil.getLen(targets);
				for(int k=0; k<targetCnt; k++) {
					TCComponent target = targets[k];
					if(!(target instanceof TCComponentItemRevision)) {
						continue;
					}
					TCComponentItemRevision meTargetRev = (TCComponentItemRevision) target;
					TSGCBean bean = new TSGCBean(meTargetRev, procRev, meopRev, tsgcRev);
					if (tsgcBeans.contains(bean)) {
						System.out.println(">> skip "+meTargetRev);
					} else {
						tsgcBeans.add(bean);
					}
				}
			}
		}
	}

	public static void parse(TCSession session, TCComponentItemRevision procRev, TCComponentItemRevision meTargetRev, List<TSGCBean> tsgcBeans) throws Exception {
		if (procRev == null || meTargetRev == null) {
			return;
		}

		TCComponentBOPWindowType bopWindowType = (TCComponentBOPWindowType) session.getTypeComponent("BOPWindow");
		TCComponentBOPWindow window = bopWindowType.createBOPWindow(null);
		try {
			TCComponentBOMLine procLine = window.setWindowTopLine(procRev.getItem(), procRev, null, null);
			AIFComponentContext[] meopLines = procLine.getChildren();
			int meopCnt = KUtil.getLen(meopLines);
			for (int i = 0; i < meopCnt; i++) {
				TCComponentBOMLine meopLine = (TCComponentBOMLine) meopLines[i].getComponent();
				TCComponentItemRevision meopRev = meopLine.getItemRevision();
				if (meopRev == null) {
					continue;
				}
				AIFComponentContext[] cc = meopLine.getChildren();
				int cnt = KUtil.getLen(cc);
				for (int j = 0; j < cnt; j++) {
					TCComponentBOMLine cLine = (TCComponentBOMLine) cc[j].getComponent();
					TCComponentItemRevision cRev = cLine.getItemRevision();
					if (cRev == null) {
						continue;
					}
					if (TSGCExportOperation.TYPE_REV_TSGC.equals(cRev.getType())) {
						System.out.println(">> " + meTargetRev + "|" + meopRev + "|" + cRev);
						TSGCBean bean = new TSGCBean(meTargetRev, procRev, meopRev, cRev);
						if (tsgcBeans.contains(bean)) {
							System.out.println(">> skip");
						} else {
							tsgcBeans.add(bean);
						}
					}
				}
			}
		} finally {
			window.setClearCacheOnClose(true);
			window.close();
		}
	}

	@Override
	public boolean equals(Object paramObject) {
		if (paramObject == null) {
			return false;
		}
		if (this == paramObject) {
			return true;
		}
		if (paramObject instanceof TSGCBean) {
			TSGCBean bean = (TSGCBean) paramObject;
			return this.meopRev.equals(bean.meopRev) && this.meTargetRev.equals(bean.meTargetRev) && this.tsgcRev.equals(bean.tsgcRev);
		}
		return false;
	}
}
