package cn.net.connor.process.tsgcexport;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.kernel.AIFComponentContext;
import com.teamcenter.rac.aif.kernel.InterfaceAIFComponent;
import com.teamcenter.rac.kernel.TCComponentBOMLine;
import com.teamcenter.rac.kernel.TCComponentBOMWindow;
import com.teamcenter.rac.kernel.TCComponentBOMWindowType;
import com.teamcenter.rac.kernel.TCComponentDataset;
import com.teamcenter.rac.kernel.TCComponentDatasetType;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCException;
import com.teamcenter.rac.kernel.TCSession;
import com.teamcenter.rac.util.Registry;

import k.KOperation2;
import k.util.KUtil;
import k.util.POIUtil;

public class TSGCExportOperation extends KOperation2 {

	private TCComponentItemRevision productRev;
	private static final Registry REG = Registry.getRegistry(TSGCExportOperation.class);
	public static final String TYPE_REV_TSGC = "Ac5_TSGCRevision";
	public static final String PREF_EXPORT = "CONNOR_AVIC_TSGC_Export"; // TITLE=proploc.propname item,rev,itemmaster.revmaster p m index
	protected List<String> headers = new ArrayList<String>();
	protected List<String> propConfigs = new ArrayList<String>();
	protected String fileName;

	public TSGCExportOperation(AbstractAIFApplication app, String opName) {
		super(app, opName);
	}

	@Override
	public boolean init() throws Exception {
		InterfaceAIFComponent target = app.getTargetComponent();
		System.out.println("select: " + target);
		if (target instanceof TCComponentItemRevision) {
			this.productRev = (TCComponentItemRevision) target;
		}
		if (this.productRev == null) {
			throw new TCException(REG.getString("selectproductrev.ERROR"));
		}
		String[] prefValues = KUtil.getPrefVals(session, PREF_EXPORT);
		int len = KUtil.getLen(prefValues);
		for (int i = 1; i < len; i++) {
			String pref = prefValues[i].trim();
			int ind = pref.indexOf('=');
			String header = null;
			String propConfig = null;
			if (ind > 0) {
				header = pref.substring(0, ind);
				propConfig = pref.substring(ind + 1);
				headers.add(header);
				propConfigs.add(propConfig);
			}
		}
		if (headers.size() == 0 || KUtil.isEmpty(fileName = prefValues[0])) {
			throw new TCException(REG.getString("preference.ERROR") + ": " + PREF_EXPORT);
		}
		try {
			String dateStr = KUtil.getDateStr(new Date(), fileName);
			this.fileName = dateStr;
		} catch (Exception e) {
		}
		return true;
	}

	@Override
	public void execute() throws Exception {
		session.setStatus(REG.getString("readproc.STATUS"));
		List<METargetBean> metargetBeans = readProcess();
		int metargetCnt = metargetBeans == null ? 0 : metargetBeans.size();
		System.out.println("find metarget cnt: " + metargetCnt);
		List<TSGCBean> tsgcBeans = new ArrayList<TSGCBean>();
		for (int i = 0; i < metargetCnt; i++) {
			session.setStatus(String.format("( %d / %d ) %s", i + 1, metargetCnt, REG.getString("readtsgc.STATUS")));
			METargetBean metargetBean = metargetBeans.get(i);
			metargetBean.readTSGC(session, tsgcBeans);
		}
		// export
		File tempFile;
		if (!fileName.toLowerCase().endsWith(".xlsx")) {
			tempFile = new File(System.getenv("temp") + File.separator + fileName + ".xlsx");
		} else {
			tempFile = new File(System.getenv("temp") + File.separator + fileName);
		}
		exportTSGC(session, tsgcBeans, headers, propConfigs, tempFile);
		TCComponentDatasetType dsType = (TCComponentDatasetType) session.getTypeComponent("MSExcelX");
		TCComponentDataset ds = dsType.create(fileName, "", "MSExcelX");
		session.getUser().getNewStuffFolder().add("contents", ds);
		KUtil.setDsFile2(ds, "excel", tempFile);
		tempFile.delete();
		KUtil.info(null, REG.getString("export.SUCCESS"));
	}

	public static void exportTSGC(TCSession session, List<TSGCBean> tsgcBeans, List<String> headers, List<String> propConfigs, File saveFile) throws Exception {
		int tsgcCnt = tsgcBeans.size();
		System.out.println("find tsgc cnt: " + tsgcCnt);
		if (tsgcCnt < 1) {
			throw new TCException(REG.getString("notsgc.ERROR"));
		}
		session.setStatus(REG.getString("export.STATUS"));
		String sheetName = saveFile.getName();
		int ind = sheetName.indexOf('.');
		if (ind > 0) {
			sheetName = sheetName.substring(0, ind);
		}
		XSSFWorkbook workbook = new XSSFWorkbook();
		XSSFSheet sheet = workbook.createSheet(sheetName);
		XSSFRow row = POIUtil.getRow(sheet, 0);
		int colCnt = headers.size();
		for (int i = 0; i < colCnt; i++) {
			POIUtil.setValue(row, i, headers.get(i));
		}
		int rowCnt = tsgcCnt;
		for (int r = 0; r < rowCnt; r++) {
			session.setStatus(String.format("( %d / %d ) %s", r + 1, rowCnt, REG.getString("export.STATUS")));
			row = POIUtil.getRow(sheet, r + 1);
			TSGCBean bean = tsgcBeans.get(r);
			for (int c = 0; c < colCnt; c++) {
				String propConfig = propConfigs.get(c);
				if ("index".equalsIgnoreCase(propConfig)) {
					XSSFCell cell = row.getCell(c);
					if (cell == null) {
						cell = row.createCell(c);
					}
					cell.setCellValue(r + 1);
				} else {
					String value = KUtil.getMarkedPropValue(propConfig, bean.markedComponents);
					POIUtil.setValue(row, c, value);
				}
			}
		}
		for (int i = 0; i < colCnt; i++) {
			sheet.autoSizeColumn(i);
			sheet.setColumnWidth(i, (int) Math.floor(sheet.getColumnWidth(i) * 1.5));
		}
		sheet.createFreezePane(0, 1);
		session.setStatus(REG.getString("savefile.STATUS"));
		System.out.println("export to " + saveFile.getAbsolutePath());
		if (saveFile.exists()) {
			saveFile.delete();
		}
		FileOutputStream os = new FileOutputStream(saveFile);
		workbook.write(os);
		os.flush();
		os.close();
	}

	private List<METargetBean> readProcess() throws Exception {
		System.out.println("read process");
		TCComponentBOMWindowType bopWindowType = (TCComponentBOMWindowType) session.getTypeComponent("BOMWindow");
		TCComponentBOMWindow window = bopWindowType.create(null);
		List<METargetBean> metargetBeans;
		try {
			TCComponentBOMLine topLine = window.setWindowTopLine(this.productRev.getItem(), this.productRev, null, null);
			metargetBeans = new ArrayList<METargetBean>();
			cycleBom(topLine, metargetBeans);
		} finally {
			window.setClearCacheOnClose(true);
			window.close();
		}
		return metargetBeans;
	}

	private void cycleBom(TCComponentBOMLine pline, List<METargetBean> metargetBeans) throws Exception {
		METargetBean bean = METargetBean.parse(pline);
		if (bean != null) {
			metargetBeans.add(bean);
		}
		AIFComponentContext[] cc = pline.getChildren();
		int childCnt = KUtil.getLen(cc);
		for (int i = 0; i < childCnt; i++) {
			TCComponentBOMLine cline = (TCComponentBOMLine) cc[i].getComponent();
			cycleBom(cline, metargetBeans);
		}
	}

	@Override
	public void clearCache() throws Exception {

	}

}
