package cn.net.connor.process.tsgcrefexport;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.filechooser.FileSystemView;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aifrcp.AIFUtility;
import com.teamcenter.rac.kernel.TCComponentContextList;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCException;
import com.teamcenter.rac.util.MessageBox;
import com.teamcenter.rac.util.Registry;

import cn.net.connor.process.tsgcexport.TSGCBean;
import cn.net.connor.process.tsgcexport.TSGCExportOperation;
import k.KDialogController;
import k.KOperation2;
import k.util.KUtil;

public class TSGCRefExportDialogController extends KDialogController {
	private static final Registry REG = Registry.getRegistry(TSGCRefExportDialogController.class);
	public static final String TYPE_REV_TSGC = "Ac5_TSGCRevision";
	private static final String PREF_EXPORT = "CONNOR_AVIC_TSGC_Ref_Export";
	protected List<String> headers = new ArrayList<String>();
	protected List<String> propConfigs = new ArrayList<String>();
	protected String fileName;
	private TSGCRefExportDialog dialog;

	public TSGCRefExportDialogController(AbstractAIFApplication app) {
		super(app);
	}

	@Override
	public boolean init() throws Exception {
		this.dialog = (TSGCRefExportDialog) aifDialog;
		String[] prefValues = KUtil.getPrefVals(session, PREF_EXPORT);
		int len = KUtil.getLen(prefValues);
		for (int i = 1; i < len; i++) {
			String pref = prefValues[i].trim();
			int ind = pref.indexOf('=');
			String header = null;
			String propConfig = null;
			if (ind > 0) {
				header = pref.substring(0, ind);
				propConfig = pref.substring(ind + 1);
				headers.add(header);
				propConfigs.add(propConfig);
			}
		}
		if (headers.size() == 0 || KUtil.isEmpty(fileName = prefValues[0])) {
			throw new TCException(REG.getString("preference.ERROR") + ": " + PREF_EXPORT);
		}
		try {
			String dateStr = KUtil.getDateStr(new Date(), fileName);
			this.fileName = dateStr;
		} catch (Exception e) {
		}
		return true;
	}

	public void doExport() throws Exception {
		String id = dialog.tf_id.getText();
		String name = dialog.tf_name.getText();
		List<String> keys = new ArrayList<String>();
		List<String> values = new ArrayList<String>();
		if (!KUtil.isEmpty(id)) {
			keys.add(KUtil.getText("ItemID"));
			values.add(id);
		}
		if (!KUtil.isEmpty(name)) {
			keys.add(KUtil.getText("Name"));
			values.add(name);
		}
		if (keys.size() == 0) {
			throw new TCException(REG.getString("noinput.ERROR"));
		}
		keys.add(KUtil.getText("Type"));
		values.add(TYPE_REV_TSGC);
		// save file path
		String defaultFile = FileSystemView.getFileSystemView().getHomeDirectory().getPath() + File.separator + fileName;
		if (!defaultFile.toLowerCase().endsWith(".xlsx")) {
			defaultFile += ".xlsx";
		}
		File saveFile = KUtil.chooseSaveFile(dialog, defaultFile, "xlsx");
		if (saveFile == null) {
			System.out.println("canceled");
			return;
		}
		dialog.disposeDialog();
		new KOperation2(AIFUtility.getCurrentApplication(), REG.getString("dialog.TITLE")) {
			@Override
			public boolean init() throws Exception {
				return true;
			}

			@Override
			public void execute() throws Exception {
				try {
					session.setStatus(REG.getString("query.STATUS"));
					List<TCComponentItemRevision> revs = doQuery(keys, values);
					int tsgcCnt = revs.size();
					System.out.println("find tsgc count: " + tsgcCnt);
					List<TSGCBean> tsgcBeans = new ArrayList<TSGCBean>();
					for (int i = 0; i < tsgcCnt; i++) {
						session.setStatus(String.format("( %d / %d ) %s", i + 1, tsgcCnt, REG.getString("parsedata.STATUS")));
						TCComponentItemRevision rev = revs.get(i);
						TSGCBean.parse(rev, tsgcBeans);
					}
					// export
					TSGCExportOperation.exportTSGC(session, tsgcBeans, headers, propConfigs, saveFile);
					// sucess info
					KUtil.info(null, REG.getString("export.SUCCESS"));
				} catch (Exception e) {
					e.printStackTrace();
					MessageBox.post(dialog, e);
				}
			}

			@Override
			public void clearCache() throws Exception {
			}
		}.executeModeless();
	}

	protected List<TCComponentItemRevision> doQuery(List<String> keys, List<String> values) throws Exception {
		List<TCComponentItemRevision> revs = new ArrayList<TCComponentItemRevision>();
		TCComponentContextList res = KUtil.query(session, "Latest Item Revision...", keys.toArray(new String[] {}), values.toArray(new String[] {}));
		int cnt = res == null ? 0 : res.getListCount();
		for (int i = 0; i < cnt; i++) {
			TCComponentItemRevision rev = (TCComponentItemRevision) res.get(i).getComponent();
			revs.add(rev);
		}
		return revs;
	}

}
